local io = io
local pairs = pairs
local ipairs = ipairs
local type = type
local tostring = tostring
local table = table
module(...)

local board = {
  ['NSG600'] = {
    description = '1Eth, 1-USB, RS-232',
    eth = 1,
    usb = 1,
    rs232 = 1,
  },
  ['NSG600D']= {
    description = '1Eth, CDMA EV-DO rev.A, 1-USB, RS-232',
    eth = 1,
    usb = 1,
    rs232 = 1,
    cdma = 1,
  },
  ['NSG600DF']= {
    description = '1Eth, CDMA EV-DO rev.A, Wi-Fi, RS-232',
    eth = 1,
    rs232 = 1,
    cdma = 1,
    wifi = 1,
  },
  ['NSG600DFi']= {
    description = '1Eth, CDMA EV-DO rev.A, Wi-Fi, RS-232, 1-Wire',
    eth = 1,
    rs232 = 1,
    cdma = 1,
    wifi = 1,
    ['1-wire'] = 1,
  },
  ['NSG600DFr']= {
    description = '1Eth, CDMA EV-DO rev.A, Wi-Fi, RS-485',
    eth = 1,
    rs485 = 1,
    cdma = 1,
    wifi = 1,
  },
  ['NSG600DFri']= {
    description = '1Eth, CDMA EV-DO rev.A, Wi-Fi, RS-485, 1-Wire',
    eth = 1,
    rs485 = 1,
    cdma = 1,
    wifi = 1,
    ['1-wire'] = 1,
  },
  ['NSG600Di']= {
    description = '1Eth, CDMA EV-DO rev.A, 1-USB, RS-232, 1-Wire',
    eth = 1,
    usb = 1,
    rs232 = 1,
    cdma = 1,
    ['1-wire'] = 1,
  },
  ['NSG600Dr']= {
    description = '1Eth, CDMA EV-DO rev.A, 1-USB, RS-485',
    eth = 1,
    usb = 1,
    rs485 = 1,
    cdma = 1,
  },
  ['NSG600Dri']= {
    description = '1Eth, CDMA EV-DO rev.A, 1-USB, RS-485, 1-Wire',
    eth = 1,
    usb = 1,
    rs485 = 1,
    cdma = 1,
    ['1-wire'] = 1,
  },
  ['NSG600DX']= {
    description = '1Eth, CDMA EV-DO rev.A, WiMAX, RS-232',
    eth = 1,
    rs232 = 1,
    cdma = 1,
    wimax = 1,
  },
  ['NSG600DXi']= {
    description = '1Eth, CDMA EV-DO rev.A, WiMAX, RS-232, 1-Wire',
    eth = 1,
    rs232 = 1,
    cdma = 1,
    wimax = 1,
    ['1-wire'] = 1,
  },
  ['NSG600DXr']= {
    description = '1Eth, CDMA EV-DO rev.A, WiMAX, RS-485',
    eth = 1,
    rs485 = 1,
    cdma = 1,
    wimax = 1,
  },
  ['NSG600DXri']= {
    description = '1Eth, CDMA EV-DO rev.A, WiMAX, RS-485, 1-Wire',
    eth = 1,
    rs485 = 1,
    cdma = 1,
    wimax = 1,
    ['1-wire'] = 1,
  },
  ['NSG600F']= {
    description = '1Eth, Wi-Fi, 1-USB, RS-232',
    eth = 1,
    usb = 1,
    rs232 = 1,
    wifi = 1,
  },
  ['NSG600Fi']= {
    description = '1Eth, Wi-Fi, 1-USB, RS-232, 1-Wire',
    eth = 1,
    usb = 1,
    rs232 = 1,
    wifi = 1,
    ['1-wire'] = 1,
  },
  ['NSG600Fr']= {
    description = '1Eth, Wi-Fi, 1-USB, RS-485',
    eth = 1,
    usb = 1,
    rs485 = 1,
    wifi = 1,
  },
  ['NSG600Fri']= {
    description = '1Eth, Wi-Fi, 1-USB, RS-485, 1-Wire',
    eth = 1,
    usb = 1,
    rs485 = 1,
    wifi = 1,
    ['1-wire'] = 1,
  },
  ['NSG600G']= {
    description = '1Eth, EDGE, 1-USB, RS-232',
    usb = 1,
    eth = 1,
    rs232 = 1,
    edge = 1,
  },
  ['NSG600GF']= {
    description = '1Eth, EDGE, Wi-Fi, RS-232',
    eth = 1,
    rs232 = 1,
    edge = 1,
    wifi = 1,
  },
  ['NSG600GFi']= {
    description = '1Eth, EDGE, Wi-Fi, RS-232, 1-Wire',
    eth = 1,
    rs232 = 1,
    edge = 1,
    wifi = 1,
    ['1-wire'] = 1,
  },
  ['NSG600GFr']= {
    description = '1Eth, EDGE, Wi-Fi, RS-485',
    eth = 1,
    rs485 = 1,
    edge = 1,
    wifi = 1,
  },
  ['NSG600GFri']= {
    description = '1Eth, EDGE, Wi-Fi, RS-485, 1-Wire',
    eth = 1,
    rs485 = 1,
    edge = 1,
    wifi = 1,
    ['1-wire'] = 1,
  },
  ['NSG600Gi']= {
    description = '1Eth, EDGE, 1-USB, RS-232, 1-Wire',
    eth = 1,
    usb = 1,
    rs232 = 1,
    edge = 1,
    ['1-wire'] = 1,
  },
  ['NSG600Gr']= {
    description = '1Eth, EDGE, 1-USB, RS-485',
    eth = 1,
    usb = 1,
    rs485 = 1,
    edge = 1,
  },
  ['NSG600Gri']= {
    description = '1Eth, EDGE, 1-USB, RS-485, 1-Wire',
    eth = 1,
    usb = 1,
    rs485 = 1,
    edge = 1,
    ['1-wire'] = 1,
  },
  ['NSG600GX']= {
    description = '1Eth, EDGE, WiMAX, RS-232',
    eth = 1,
    rs232 = 1,
    edge = 1,
    wimax = 1,
  },
  ['NSG600GXi']= {
    description = '1Eth, EDGE, WiMAX, RS-232, 1-Wire',
    eth = 1,
    rs232 = 1,
    edge = 1,
    wimax = 1,
    ['1-wire'] = 1,
  },
  ['NSG600GXr']= {
    description = '1Eth, EDGE, WiMAX, RS-485',
    eth = 1,
    rs485 = 1,
    edge = 1,
    wimax = 1,
  },
  ['NSG600GXri']= {
    description = '1Eth, EDGE, WiMAX, RS-485, 1-Wire',
    eth = 1,
    rs485 = 1,
    edge = 1,
    wimax = 1,
    ['1-wire'] = 1,
  },
  ['NSG600H']= {
    description = '1Eth, 3G, 1-USB, RS-232',
    eth = 1,
    usb = 1,
    rs232 = 1,
    ['3g'] = 1,
  },
  ['NSG600HF']= {
    description = '1Eth, 3G, Wi-Fi, RS-232',
    eth = 1,
    rs232 = 1,
    ['3g'] = 1,
    wifi = 1,
  },
  ['NSG600HFi']= {
    description = '1Eth, 3G, Wi-Fi, RS-232, 1-Wire',
    eth = 1,
    rs232 = 1,
    ['3g'] = 1,
    wifi = 1,
    ['1-wire'] = 1,
  },
  ['NSG600HFr']= {
    description = '1Eth, 3G, Wi-Fi, RS-485',
    eth = 1,
    rs485 = 1,
    ['3g'] = 1,
    wifi = 1,
  },
  ['NSG600HFri']= {
    description = '1Eth, 3G, Wi-Fi, RS-485, 1-Wire',
    eth = 1,
    rs485 = 1,
    ['3g'] = 1,
    wifi = 1,
    ['1-wire'] = 1,
  },
  ['NSG600Hi']= {
    description = '1Eth, 3G, 1-USB, RS-232, 1-Wire',
    eth = 1,
    usb = 1,
    rs232 = 1,
    ['3g'] = 1,
    ['1-wire'] = 1,
  },
  ['NSG600Hr']= {
    description = '1Eth, 3G, 1-USB, RS-485',
    eth = 1,
    usb = 1,
    rs485 = 1,
    ['3g'] = 1,
  },
  ['NSG600Hri']= {
    description = '1Eth, 3G, 1-USB, RS-485, 1-Wire',
    eth = 1,
    usb = 1,
    rs485 = 1,
    ['3g'] = 1,
    ['1-wire'] = 1,
  },
  ['NSG600HX']= {
    description = '1Eth, 3G, WiMAX, RS-232',
    eth = 1,
    rs232 = 1,
    ['3g'] = 1,
    wimax = 1,
  },
  ['NSG600HXi']= {
    description = '1Eth, 3G, WiMAX, RS-232, 1-Wire',
    eth = 1,
    rs232 = 1,
    ['3g'] = 1,
    wimax = 1,
    ['1-wire'] = 1,
  },
  ['NSG600HXr']= {
    description = '1Eth, 3G, WiMAX, RS-485',
    eth = 1,
    rs485 = 1,
    ['3g'] = 1,
    wimax = 1,
  },
  ['NSG600HXri']= {
    description = '1Eth, 3G, WiMAX, RS-485, 1-Wire',
    eth = 1,
    rs485 = 1,
    ['3g'] = 1,
    wimax = 1,
    ['1-wire'] = 1,
  },
  ['NSG600i']= {
    description = '1Eth, 1-USB, RS-232, 1-Wire',
    eth = 1,
    usb = 1,
    rs232 = 1,
    ['1-wire'] = 1,
  },
  ['NSG600r']= {
    description = '1Eth, 1-USB, RS-485',
    usb = 1,
    eth = 1,
    rs485 = 1,
  },
  ['NSG600ri']= {
    description = '1Eth, 1-USB, RS-485, 1-Wire',
    eth = 1,
    usb = 1,
    rs485 = 1,
    ['1-wire'] = 1,
  },
  ['NSG600X']= {
    description = '1Eth, WiMAX, 1-USB, RS-232',
    eth = 1,
    usb = 1,
    rs232 = 1,
    wimax = 1,
  },
  ['NSG600Xi']= {
    description = '1Eth, WiMAX, 1-USB, RS-232, 1-Wire',
    eth = 1,
    usb = 1,
    rs232 = 1,
    wimax = 1,
    ['1-wire'] = 1,
  },
  ['NSG600Xr']= {
    description = '1Eth, WiMAX, 1-USB, RS-485',
    eth = 1,
    usb = 1,
    rs485 = 1,
    wimax = 1,
  },
  ['NSG600Xri']= {
    description = '1Eth, WiMAX, 1-USB, RS-485, 1-Wire',
    eth = 1,
    usb = 1,
    rs485 = 1,
    wimax = 1,
    ['1-wire'] = 1,
  },
  ['NSG605']= {
    description = '5Eth, 1-USB, RS-232',
    eth = 5,
    usb = 1,
    rs232 = 1,
  },
  ['NSG605D']= {
    description = '5Eth, CDMA EV-DO rev.A, 1-USB, RS-232',
    eth = 5,
    usb = 1,
    rs232 = 1,
    cdma = 1,
  },
  ['NSG605DF']= {
    description = '5Eth, CDMA EV-DO rev.A, Wi-Fi, RS-232',
    eth = 5,
    rs232 = 1,
    cdma = 1,
    wifi = 1,
  },
  ['NSG605DFi']= {
    description = '5Eth, CDMA EV-DO rev.A, Wi-Fi, RS-232, 1-Wire',
    eth = 5,
    rs232 = 1,
    cdma = 1,
    wifi = 1,
    ['1-wire'] = 1,
  },
  ['NSG605DFr']= {
    description = '5Eth, CDMA EV-DO rev.A, Wi-Fi, RS-485',
    eth = 5,
    rs485 = 1,
    cdma = 1,
    wifi = 1,
  },
  ['NSG605DFri']= {
    description = '5Eth, CDMA EV-DO rev.A, Wi-Fi, RS-485, 1-Wire',
    eth = 5,
    rs485 = 1,
    cdma = 1,
    wifi = 1,
    ['1-wire'] = 1,
  },
  ['NSG605Di']= {
    description = '5Eth, CDMA EV-DO rev.A, 1-USB, RS-232, 1-Wire',
    eth = 5,
    usb = 1,
    rs232 = 1,
    cdma = 1,
    ['1-wire'] = 1,
  },
  ['NSG605Dr']= {
    description = '5Eth, CDMA EV-DO rev.A, 1-USB, RS-485',
    eth = 5,
    usb = 1,
    rs485 = 1,
    cdma = 1,
  },
  ['NSG605Dri']= {
    description = '5Eth, CDMA EV-DO rev.A, 1-USB, RS-485, 1-Wire',
    eth = 5,
    usb = 1,
    rs485 = 1,
    cdma = 1,
    ['1-wire'] = 1,
  },
  ['NSG605DX']= {
    description = '5Eth, CDMA EV-DO rev.A, WiMAX, RS-232',
    eth = 5,
    rs232 = 1,
    cdma = 1,
    wimax = 1,
  },
  ['NSG605DXi']= {
    description = '5Eth, CDMA EV-DO rev.A, WiMAX, RS-232, 1-Wire',
    eth = 5,
    rs232 = 1,
    cdma = 1,
    wimax = 1,
    ['1-wire'] = 1,
  },
  ['NSG605DXr']= {
    description = '5Eth, CDMA EV-DO rev.A, WiMAX, RS-485',
    eth = 5,
    rs485 = 1,
    cdma = 1,
    wimax = 1,
  },
  ['NSG605DXri']= {
    description = '5Eth, CDMA EV-DO rev.A, WiMAX, RS-485, 1-Wire',
    eth = 5,
    rs485 = 1,
    cdma = 1,
    wimax = 1,
    ['1-wire'] = 1,
  },
  ['NSG605F']= {
    description = '5Eth, Wi-Fi, 1-USB, RS-232',
    eth = 5,
    usb = 1,
    rs232 = 1,
    wifi = 1,
  },
  ['NSG605Fi']= {
    description = '5Eth, Wi-Fi, 1-USB, RS-232, 1-Wire',
    eth = 5,
    usb = 1,
    rs232 = 1,
    wifi = 1,
    ['1-wire'] = 1,
  },
  ['NSG605Fr']= {
    description = '5Eth, Wi-Fi, 1-USB, RS-485',
    eth = 5,
    usb = 1,
    rs485 = 1,
    wifi = 1,
  },
  ['NSG605Fri']= {
    description = '5Eth, Wi-Fi, 1-USB, RS-485, 1-Wire',
    eth = 5,
    usb = 1,
    rs485 = 1,
    wifi = 1,
    ['1-wire'] = 1,
  },
  ['NSG605G']= {
    description = '5Eth, EDGE, 1-USB, RS-232',
    eth = 5,
    usb = 1,
    rs232 = 1,
    edge = 1,
  },
  ['NSG605GF']= {
    description = '5Eth, EDGE, Wi-Fi, RS-232',
    eth = 5,
    rs232 = 1,
    edge = 1,
    wifi = 1,
  },
  ['NSG605GFi']= {
    description = '5Eth, EDGE, Wi-Fi, RS-232, 1-Wire',
    eth = 5,
    rs232 = 1,
    edge = 1,
    wifi = 1,
    ['1-wire'] = 1,
  },
  ['NSG605GFr']= {
    description = '5Eth, EDGE, Wi-Fi, RS-485',
    eth = 5,
    rs485 = 1,
    edge = 1,
    wifi = 1,
  },
  ['NSG605GFri']= {
    description = '5Eth, EDGE, Wi-Fi, RS-485, 1-Wire',
    eth = 5,
    rs485 = 1,
    edge = 1,
    wifi = 1,
    ['1-wire'] = 1,
  },
  ['NSG605Gi']= {
    description = '5Eth, EDGE, 1-USB, RS-232, 1-Wire',
    eth = 5,
    usb = 1,
    rs232 = 1,
    edge = 1,
    ['1-wire'] = 1,
  },
  ['NSG605Gr']= {
    description = '5Eth, EDGE, 1-USB, RS-485',
    eth = 5,
    usb = 1,
    rs485 = 1,
    edge = 1,
  },
  ['NSG605Gri']= {
    description = '5Eth, EDGE, 1-USB, RS-485, 1-Wire',
    usb = 1,
    eth = 5,
    rs485 = 1,
    edge = 1,
    ['1-wire'] = 1,
  },
  ['NSG605GX']= {
    description = '5Eth, EDGE, WiMAX, RS-232',
    eth = 5,
    rs232 = 1,
    edge = 1,
    wimax = 1,
  },
  ['NSG605GXi']= {
    description = '5Eth, EDGE, WiMAX, RS-232, 1-Wire',
    eth = 5,
    rs232 = 1,
    edge = 1,
    wimax = 1,
    ['1-wire'] = 1,
  },
  ['NSG605GXr']= {
    description = '5Eth, EDGE, WiMAX, RS-485',
    eth = 5,
    rs485 = 1,
    edge = 1,
    wimax = 1,
  },
  ['NSG605GXri']= {
    description = '5Eth, EDGE, WiMAX, RS-485, 1-Wire',
    eth = 5,
    rs485 = 1,
    edge = 1,
    wimax = 1,
    ['1-wire'] = 1,
  },
  ['NSG605H']= {
    description = '5Eth, 3G, 1-USB, RS-232',
    eth = 5,
    usb = 1,
    rs232 = 1,
    ['3g'] = 1,
  },
  ['NSG605HF']= {
    description = '5Eth, 3G, Wi-Fi, RS-232',
    eth = 5,
    rs232 = 1,
    ['3g'] = 1,
    wifi = 1,
  },
  ['NSG605HFi']= {
    description = '5Eth, 3G, Wi-Fi, RS-232, 1-Wire',
    eth = 5,
    rs232 = 1,
    ['3g'] = 1,
    wifi = 1,
    ['1-wire'] = 1,
  },
  ['NSG605HFr']= {
    description = '5Eth, 3G, Wi-Fi, RS-485',
    eth = 5,
    rs485 = 1,
    ['3g'] = 1,
    wifi = 1,
  },
  ['NSG605HFri']= {
    description = '5Eth, 3G, Wi-Fi, RS-485, 1-Wire',
    eth = 5,
    rs485 = 1,
    ['3g'] = 1,
    wifi = 1,
    ['1-wire'] = 1,
  },
  ['NSG605Hi']= {
    description = '5Eth, 3G, 1-USB, RS-232, 1-Wire',
    eth = 5,
    usb = 1,
    rs232 = 1,
    ['3g'] = 1,
    ['1-wire'] = 1,
  },
  ['NSG605Hr']= {
    description = '5Eth, 3G, 1-USB, RS-485',
    eth = 5,
    usb = 1,
    rs485 = 1,
    ['3g'] = 1,
  },
  ['NSG605Hri']= {
    description = '5Eth, 3G, 1-USB, RS-485, 1-Wire',
    eth = 5,
    usb = 1,
    rs485 = 1,
    ['3g'] = 1,
    ['1-wire'] = 1,
  },
  ['NSG605HX']= {
    description = '5Eth, 3G, WiMAX, RS-232',
    eth = 5,
    rs232 = 1,
    ['3g'] = 1,
    wimax = 1,
  },
  ['NSG605HXi']= {
    description = '5Eth, 3G, WiMAX, RS-232, 1-Wire',
    eth = 5,
    rs232 = 1,
    ['3g'] = 1,
    wimax = 1,
    ['1-wire'] = 1,
  },
  ['NSG605HXr']= {
    description = '5Eth, 3G, WiMAX, RS-485',
    eth = 5,
    rs485 = 1,
    ['3g'] = 1,
    wimax = 1,
  },
  ['NSG605HXri']= {
    description = '5Eth, 3G, WiMAX, RS-485, 1-Wire',
    eth = 5,
    rs485 = 1,
    ['3g'] = 1,
    wimax = 1,
    ['1-wire'] = 1,
  },
  ['NSG605i']= {
    description = '5Eth, 1-USB, RS-232, 1-Wire',
    eth = 5,
    usb = 1,
    rs232 = 1,
    ['1-wire'] = 1,
  },
  ['NSG605r']= {
    description = '5Eth, 1-USB, RS-485',
    eth = 5,
    usb = 1,
    rs485 = 1,
  },
  ['NSG605ri']= {
    description = '5Eth, 1-USB, RS-485, 1-Wire',
    eth = 5,
    usb = 1,
    rs485 = 1,
    ['1-wire'] = 1,
  },
  ['NSG605X']= {
    description = '5Eth, WiMAX, 1-USB, RS-232',
    eth = 5,
    usb = 1,
    rs232 = 1,
    wimax = 1,
  },
  ['NSG605Xi']= {
    description = '5Eth, WiMAX, 1-USB, RS-232, 1-Wire',
    eth = 5,
    usb = 1,
    rs232 = 1,
    wimax = 1,
    ['1-wire'] = 1,
  },
  ['NSG605Xr']= {
    description = '5Eth, WiMAX, 1-USB, RS-485',
    eth = 5,
    usb = 1,
    rs485 = 1,
    wimax = 1,
  },
  ['NSG605Xri']= {
    description = '5Eth, WiMAX, 1-USB, RS-485, 1-Wire',
    eth = 5,
    usb = 1,
    rs485 = 1,
    wimax = 1,
    ['1-wire'] = 1,
  },
}

deviceLongName = {
  eth = 'Eth',
  usb = 'USB',
  rs232 = 'RS-232',
  rs485 = 'RS-485',
  ['3g'] = '3G',
  edge = 'EDGE',
  cdma = 'CDMA EV-DO',
  wimax = 'WiMAX',
  wifi = 'Wi-Fi',
  ['1-wire'] = '1-Wire',
}
deviceList = {'eth', 'usb', 'rs232', 'rs485', '3g', 'edge', 'cdma',
'wimax', 'wifi', '1-wire'}

function set(bi)
  local f = io.open("/proc/nsg/env")
  local envStr, env = "", {}
  if f then
    envStr = f:read('*a')
    f:close()
  end
  for k, v in envStr:gmatch"%s*([^%s=]*)%s*=([^\n]*)\n" do
    if not env[k] then
      env[k] = v
    end
  end
  bi.env = env
  bi.serial = env.serial_num
  bi.ethaddr = env.ethaddr
  local mach = env.nsg_device
  if mach then
    bi.mach = mach
    if board[mach] then
      for k, v in pairs(board[mach]) do
        bi[k] = v
      end
    end
  end
  
  bi.bridge = true
  if bi.eth > 1 then
    local t = {{}}
    for i = 0, bi.eth - 1 do
      t[1]["eth"..i] = true
    end
    bi.invalidBridgeGroup = t
  end

  local info = {}
  for _, dev in ipairs(deviceList) do
    local val = bi[dev]
    if val then
      local num, name = '', deviceLongName[dev] or tostring(dev)
      if type(val) == 'number' and val > 1 then
        num = tostring(val)..' '
      end
      info[#info+1] = num..name
    end
  end
  bi.ports = table.concat(info, ', ')
  bi.info = ([[
Device: %s
Ports:  %s
SN:     %s
MAC:    %s
]]):format(bi.mach, bi.ports, bi.serial, bi.ethaddr)
  return bi
end
