module(..., package.seeall)
local s = require'scheme'
----------------------------------------

--[====[

local function WriteIfSet(target, keyword, value)
   if target and value and value ~= "0.0.0.0" then
      target[#target+1] = keyword .. value
   end
end
 
function createDhcpConfFile(conf, filename, path)
   local DhcpConf = {}

   DhcpConf[#DhcpConf+1] = ("interface %s"):format(conf.interface)

   DhcpConf[#DhcpConf+1] = ("start %s"):format(conf["ip-address-pool"].from)
   DhcpConf[#DhcpConf+1] = ("end %s"):format(conf["ip-address-pool"].to)
   DhcpConf[#DhcpConf+1] = ("opt subnet %s"):format(conf["mask"])
   DhcpConf[#DhcpConf+1] = ("opt lease  %d"):format(conf["lease-time"])
   DhcpConf[#DhcpConf+1] = ("max_leases %d"):format(conf["max-leases"]) 

   WriteIfSet(DhcpConf, "opt router ", conf["gateway"])
   WriteIfSet(DhcpConf, "opt dns ", conf["dns1"])
   WriteIfSet(DhcpConf, "opt dns ", conf["dns2"])
   
   
   local DhcpConfText = table.concat(DhcpConf, '\n')..'\n'
   local f = io.open(filename, 'w')
   if f then
      f:write(DhcpConfText)
      f:close()
   end
end
----------------------------------------
local function dhcpAtStart(oper)
   -- TODO: Its better to check interface status !!
   C.threadSleep(3)  -- Now is simple delay
   
end
-----
local function restartDhcpServer(path, sch, conf, prevConf)
   local conf = s.addDefault(path, sch, conf, true) --clone conf with defaults

   if conf.interface == "" then
      -- If iface is empty => may be "path[#path]" suited ?! 
      -- Get ifconfig this "path[#path]" and If It is good => use It (Later!) 
      return nil, 'Nothing done!  ("interface" - empty)'
   end

   local filename = os.tmpname()

   createDhcpConfFile(conf, filename, path)
   local ApCmdStart = {'/usr/sbin/udhcpd', filename }
   
   if conf["adm-state"] == 'up' then
      if conf["gateway"] ~= "0.0.0.0" then
	 s.readpopen("ifconfig wlan0 " .. conf["gateway"] .." 2>&1")
      end
      C.restartDaemon(path, ApCmdStart, 
		      { 
			 tmpFileList = {filename}, 
			 atStart = dhcpAtStart,
		      }
		   )
      return true
   end
   C.restartDaemon(path)
   return true
end
----------------------------------------
DhcpScheme = s.newScheme {
   _H = "Dhcp",
   _NAME = s.newScheme {
      
   _Z = restartDhcpServer,

      ["adm-state"] = s.newScheme{
	 _T = "choice",
	 _A = {"up", "down"},
	 _V = s.choiceVal,
	 _H = "DhcpAdmState",
	 _D = "down",
      },

      gateway = s.ipAddressType{_H = "DhcpGateway", _D = "172.16.0.1", },

      dns1 = s.ipAddressType{_H = "DhcpDns1", },

      dns2 = s.ipAddressType{_H = "DhcpDns2", },

      ["ip-address-pool"] = s.IpAddressPool,
      
      ["mask"] = s.ipAddressType{_H = "DhcpMask", _D = "255.255.0.0", },

      ["lease-time"] = s.integerType{_H = "DhcpLeaseTime", _A = {1, 4294967295}, _D = 84600, },

      ["max-leases"] = s.integerType{_H = "DhcpMaxLeases", _A = {1, 65535}, _D = 254, },

      interface = s.stringType{_H = "DhcpInterface", _D = "", },
   },

}
--]====]


local function WriteIfSet(target, keyword, value)
   if target and value and value ~= "0.0.0.0" then
      target[#target+1] = keyword .. value
   end
end
 
function createConfFile(conf, sch, filename, path)
   local ret = false
   local Conf = {}
   path[#path +1] = "dhcp"
   if type(conf["dhcp"]) == 'table' then
      local sch = C.getSchemeByPath(path)
      for k,v in pairs(conf["dhcp"] or {}) do
         if (not (sch[k] or (tostring(k)):find('^[_].*$'))) and type(v)=="table" and v["adm-state"]=="up" then
            ret = true
            Conf[#Conf +1] = table.concat{
               "dhcp-range=interface:",
               v.interface,
               ",set:",
               k,
               ",",
               v["ip-address-pool"].from,
               ",",
               v["ip-address-pool"].to,
               ",",
               v.mask,
               ",",
               v["lease-time"]
            }
            if  v.dns1 or  v.dns2 then
               Conf[#Conf +1] = table.concat{
                  "dhcp-option=tag:",
                  k,
                  ",",
                  "6",
                  v.dns1 and (","..v.dns1) ,
                  v.dns2 and (","..v.dns2),
               }
            end
            if v.gateway then 
               Conf[#Conf +1] = table.concat{
                  "dhcp-option=tag:",
                  k,
                  ",",
                  "3,",
                  v.gateway
               }
            end
         end
      end
      Conf[#Conf +1] = conf["dhcp"] and conf["dhcp"].extra or ""
      Conf[#Conf +1] = table.concat{
         "dhcp-lease-max=",
         conf["dhcp"]["max-leases"]
      }
   end
   path[#path] = "dns"
   if type(conf["dns"]) == 'table' then
      local sch = C.getSchemeByPath(path)
      for k,v in pairs(conf["dns"] or {}) do
         if (not (sch[k] or (tostring(k)):find('^[_].*$'))) and v["adm-state"]=="up" and v.interface then
            ret = true
            Conf[#Conf +1] = table.concat{
               "interface=",
               v.interface
            }
         end
      end
      if  conf["dns"].dns1 then
         Conf[#Conf +1] = table.concat{
            "server=",
            conf["dns"].dns1
         }
      end
      if conf["dns"].dns2 then 
         Conf[#Conf +1] = table.concat{
            "server=",
            conf["dns"].dns2
         }
      end
      Conf[#Conf +1] = conf["dns"] and conf["dns"].extra or ""
   end
   Conf = table.concat(Conf, '\n')..'\n'
   local f = io.open(filename, 'w')
   if f then
      f:write(Conf)
      f:close()
   end
   return ret
end

local function getParent(path, sesNum)
  local path = s.clonePath(path, -1)
  local sch = C.getSchemeByPath(path)
  local conf = C.getConfByPath(path) or {}
  local locConf = C.getConfByPath(path, sesNum) or {}
  for k, v in pairs(locConf) do
    conf[k] = v
  end
  conf = s.addDefault(path, sch, conf, true) --clone conf with defaults
  return sch, conf, path
end


local function restartDnsMasqServer(path, sch, conf, prevConf)
   local pPath
   sch, conf, pPath = getParent(path)
   path[#path] = "dhcp"

   local confFile = C.getOption(path,"confFile")
   local leaseFile = C.getOption(path,"leaseFile")
   if not confFile then
      confFile = os.tmpname()
      C.setOption(path, "confFile", confFile)
   end
   if not leaseFile then
      leaseFile = os.tmpname()
      C.setOption(path, "leaseFile", leaseFile)
   end
   
   C.restartDaemon(path)
   if createConfFile(conf,sch, confFile, pPath) then
      C.restartDaemon(path, 
                      {"/usr/sbin/dnsmasq", "--no-daemon", "--conf-file="..confFile,"--dhcp-leasefile="..leaseFile},
                      {
                         noRestart = false,
                         atExit = C.increasingSleep,
                         atKill = function()
                                     os.execute("/usr/bin/killall -SIGINT dnsmasq")
                                  end,
                      }
                   )
      return true
   else
      return true
   end
end






DhcpScheme = s.newScheme {
   _H = "Dhcp",
   _Z = restartDnsMasqServer,
   show = s.newScheme{
      _H = "DnsMasqShow",
      ["log"] =  s.newScheme{
         _H = "DnsMasqShowLog",
         _X = function(path)
                 path = s.clonePath(path, -3)
                 path[#path +1] = "dhcp"
                 return C.getDaemonLog(path)
              end,
         _R = 'user',
      },
      ["leases"] =  s.newScheme{
         _H = "DnsMasqShowLeases",
         _X = function(path)
                 path = s.clonePath(path, -3)
                 path[#path +1] = "dhcp"
                 local fname = C.getOption(path,"leaseFile")
                 local leases = s.readpopen("cat "..fname.." 2>&1")
                 return leases
              end,
         _R = 'user',
      },
      ["configuration"] =  s.newScheme{
         _H = "DnsMasqShowConfiguration",
         _X = function(path)
                 path = s.clonePath(path, -3)
                 path[#path +1] = "dhcp"
                 local fname = C.getOption(path,"confFile")
                 local leases = s.readpopen("cat "..fname.." 2>&1")
                 return leases
              end,
         _R = 'user',
      },
   },
   ["max-leases"] = s.integerType{_H = "DhcpMaxLeases", _A = {1, 65535}, _D = 254, },
   extra = s.stringType{_H = "DhcpServerExtra", _P = -100},
   _NAME = s.newScheme {
      _H = "DhcpN",
      description = s.stringType{_H = "DhcpServerNDescription", _P = 100},
      ["adm-state"] = s.newScheme{
         _T = "choice",
         _A = {"up", "down"},
         _V = s.choiceVal,
         _H = "DhcpAdmState",
         _D = "down",
      },
      
      gateway = s.ipAddressType{_H = "DhcpGateway", _D = "172.16.0.1", },

      dns1 = s.ipAddressType{_H = "DhcpDns1", },

      dns2 = s.ipAddressType{_H = "DhcpDns2", },

      ["ip-address-pool"] = s.IpAddressPool,
      
      ["mask"] = s.ipAddressType{_H = "DhcpMask", _D = "255.255.0.0", },

      ["lease-time"] = s.integerType{_H = "DhcpLeaseTime", _A = {1, 4294967295}, _D = 84600, },

      interface = s.stringType{_H = "DhcpInterface", _D = "", },
   },

}

DnsServerScheme = s.newScheme {
   _H = "DnsMasqServer",
   _Z = restartDnsMasqServer,show = s.newScheme{
      _H = "DnsMasqShow",
      ["log"] =  s.newScheme{
         _H = "DnsMasqShowLog",
         _X = function(path)
                 path = s.clonePath(path, -3)
                 path[#path +1] = "dhcp"
                 return C.getDaemonLog(path)
              end,
         _R = 'user',
      },
      ["configuration"] =  s.newScheme{
         _H = "DnsMasqShowConfiguration",
         _X = function(path)
                 path = s.clonePath(path, -3)
                 path[#path +1] = "dhcp"
                 local fname = C.getOption(path,"confFile")
                 local leases = s.readpopen("cat "..fname.." 2>&1")
                 return leases
              end,
         _R = 'user',
      },
   }, 
   dns1 = s.ipAddressType{_H = "DnsMasqDns1", },
   dns2 = s.ipAddressType{_H = "DnsMasqDns2", },
   extra = s.stringType{_H = "DnsMasqExtra", _P = -100},
   _NAME = s.newScheme {
      _H = "DnsMasqServerN",
      description = s.stringType{_H = "DnsMasqServerNDescription", _P = 100},
      ["adm-state"] = s.newScheme{
         _T = "choice",
         _A = {"up", "down"},
         _V = s.choiceVal,
         _H = "DnsMasqAdmState",
         _D = "down",
      },
      interface = s.stringType{_H = "DnsMasqInterface", _D = "", },
   },

}