module(..., package.seeall)
local s = require'scheme'
local iface_mod = require'nsgifaceconf'
local nf = require'nsgnfconf'
local bird = require'nsgbird'

local function idiv(a,b)
  return math.floor(a/b), a%b
end

local function intToIp(n, p)
  n, a4 = idiv(n,256)
  n, a3 = idiv(n,256)
  n, a2 = idiv(n,256)
  n, a1 = idiv(n,256)
  return tostring(a1)..'.'..tostring(a2)..'.'..tostring(a3)..'.'..tostring(a4)..
  (p and '/'..tostring(p) or '')
end

local function ipToInt(prefix)
  local n
  local a1, a2, a3, a4, p =
  string.match(prefix,
  "^(%d%d?%d?)%.(%d%d?%d?)%.(%d%d?%d?)%.(%d%d?%d?)(/?%d?%d?)$")
  if a1 then
    a1 = tonumber(a1)
    a2 = tonumber(a2)
    a3 = tonumber(a3)
    a4 = tonumber(a4)
    if a1 <= 255 and a2 <= 255 and a3 <= 255 and a4 <= 255 then
      n = a1 * 256 * 256 * 256 +
      a2 * 256 * 256 +
      a3 * 256 +
      a4
    else
      return nil
    end
    if p == '' then
      return n, 32
    end
    if #p > 2 and p:sub(1,1) == '/' then
      p = tonumber(p:sub(2))
      if p <= 32 then
        return n, p
      end
    end
  end
  return nil
end

local function prefixToNet(prefix)
  local n, p = ipToInt(prefix)
  if n then
    local pp = 2^(32-p)
    local net = math.floor(n/pp) * pp
    return intToIp(net, p)
  end
  return nil
end


local ip = "/usr/sbin/ip"

local function ipRouteApplyAux(path, sch, conf)
  os.execute(ip.." route flush proto nsg  >/dev/null 2>&1")
  local script = {}
  for _, route in ipairs(conf) do
    if route.network then
      local gateway = route.gateway and "via "..route.gateway or ''
      local device = route.device and "dev "..iface_mod.getIfaceName(route.device) or ''
      local metric = route.metric and "metric "..tostring(route.metric) or ''
      local com = (ip.." route add %s %s %s %s proto nsg  >/dev/null 2>&1"):
      format(
        prefixToNet(route.network) or 'default', gateway, device, metric
      )
      os.execute(com)  
      script[#script+1] = com
    end
  end
  if #script > 0 then
    local scriptName = os.tmpname()
    local f = io.open(scriptName, 'w')
    f:write(
      "while true; do sleep 300\n"..
      table.concat(script, '\n')..
      "\ndone\n" 
    )
    f:close()
    C.restartDaemon(path,
      {"sh", scriptName},
      {tmpFileList = {scriptName},}
    )
  else
    C.restartDaemon(path)
  end
end

local ipRouteAliasHookKey = {}
local ipNeteventKey = {}
local function ipRouteApply(path, sch, conf, prevConf)
  if not s.confCmp(conf, prevConf) then  
    path = s.clonePath(path)
    conf = s.addDefault(path, sch, conf or {}, true)
    iface_mod.setAliasHook(ipRouteAliasHookKey, 
      function()
        return  ipRouteApplyAux(path, sch, conf)
      end
    )
    C.updateNeteventCallback(
      ipNeteventKey,
      function(msg)
        if (msg.type == 'newlink' and msg.ifup) or msg.type == 'newaddr' then
          return  ipRouteApplyAux(path, sch, conf)
        end
      end
    )
    ipRouteApplyAux(path, sch, conf)
  end
  return true
end

local routeScheme = s.newScheme{
  _H = "ipRouteList",
  _Z = ipRouteApply,
  _LIST = s.newScheme{
    _H = "ipRoute",
    network = s.ipPrefixType{_H = "ipRouteNet", _D = '0.0.0.0/0'},
    gateway = s.ipAddressType{_H = "ipRouteVia"},
    device = s.stringType{_H = "ipRouteDev"},
    metric = s.integerType{
      _H = "ipRouteMetric",
      _A = {0, 0x7fffffff},
      _D = 0,
    }
  },
  show = s.newScheme{
    _H = "ipRouteShow",
    _X = function()
      return('\n'..s.readpopen(ip.." route show 2>&1"))
    end
  }
}

ipScheme = s.newScheme{
  _H = "ip",
  _Z = function() return true end,
  interface = iface_mod.interfaceListScheme,
  route = routeScheme{_P = 100},
  nat = nf.natScheme{_P = 20},
  filter = nf.filterScheme{_P = 10},
  ["dynamic-routing"] = bird.birdScheme{_P = 90},
  show = nf.showNetFilter,
}
