module(..., package.seeall)
local s = require'scheme'
local dhcp = require'nsgdhcp'
local netping = require'nsgnetping'
local daemons = require'nsgdaemons'

local function restartSSHDaemon(path, sch, conf, prevConf)
  if not s.confCmp(conf, prevConf) then
    conf = conf or {}
    if conf.enable then
      local sshconf = s.addDefault(path, sch, conf, true) --clone conf with defaults
      local com = {
        "/sbin/sshd",
        "-D",
        "-e",
        "-p", tostring(sshconf.port or 22),
        sshconf["debug-level"] > 0 and '-'..("d"):rep(sshconf["debug-level"])
        or nil -- must be last element in this table
      }
      C.restartDaemon(path, com, {atExit = C.increasingSleep})
    else
      C.restartDaemon(path)
    end
  end
  return true
end

local function restartHttpDaemon(path, sch, conf, prevConf, first, sesNum)
   local si = sesNum and C.getSessionInfo(sesNum)
   local needCert = 0 == os.execute('cd /etc/stunnel/;[ ! -s https_auto.key ] || [ ! -s https_auto.cert ]')
   local httpconf = s.addDefault(path, sch, conf, true) --clone conf with defaults
   if (not s.confCmp(conf, prevConf)) or
   ((needCert and httpconf.enable and httpconf.https=="enable")) then
      if si == nil or (si and si.mode ~= "html") or (si and si.EXTRA["http-apply-warn"]) then
         if httpconf.enable then
            if httpconf.https=="enable" then
               local certGenScript= [[  cd /etc/stunnel/;
                     {
                        if [ ! -s https_auto.key ] || [ ! -s https_auto.cert ];
                        then
                              /bin/openssl req -new -x509 -days 3650 -nodes -out https_auto.cert -keyout https_auto.key -config /etc/ssl/openssl.cnf -subj /C=RU/ST=Russia/L=`hostname`/O=`hostname`/OU=IT/CN=`hostname`/ ;chmod 600  https_auto.key;
                        else
                           echo 'cert and key for https already exist, skip generation'
                           fi; } 2>/dev/null;
                        exit 0;
                  ]]
               C.restartDaemon(
                  {"services","https"},
                  {"/bin/sh", "-c", certGenScript},
                  {noRestart = true}
               )
            end

            --[[
            local com = {
               "/usr/sbin/httpd",
               "-f",
               "-h"..httpconf.home or "-h/usr/share/www/",
               httpconf.port and '-p'..tonumber(httpconf.port) or nil
            }
            --]]
            local com = {
               "/usr/bin/xavante_start",
            }
            C.fakeLastComTimestamp()
            C.restartDaemon(path, com, {noTimestamp = true, atExit = C.increasingSleep}) 
         else
            C.fakeLastComTimestamp()
            C.restartDaemon(path)
         end
         
         return true
      else
         si.EXTRA["http-apply-warn"] = true
         return nil,  s.getManual({ _H = "servicesHttpApplyWarn"}, si and si.locale)
      end
      
   end
   return true
end

local function restartInetd(path, sch, conf, prevConf)
   local inetdConf = ""
   conf = s.addDefault(path, sch, conf, true) or {}
   if conf.telnet and conf.telnet.enable then
      inetdConf = inetdConf..
         ("%d stream tcp nowait root /usr/sbin/telnetd telnetd\n"):
      format(conf.telnet.port)
   end

   if conf.echo and conf.echo.tcp == "enable" then
      inetdConf = inetdConf..
         ("echo    stream  tcp     nowait  root    internal\n")
   end
   if conf.echo and conf.echo.udp=="enable" then
      inetdConf = inetdConf..
         ("echo    dgram   udp     wait    root    internal\n")
   end

   if conf.discard and conf.discard.tcp=="enable" then
      inetdConf = inetdConf..
         ("discard    stream  tcp     nowait  root    internal\n")
   end
   if conf.discard and conf.discard.udp=="enable" then
      inetdConf = inetdConf..
         ("discard    dgram   udp     wait    root    internal\n")
   end

   if conf.daytime and conf.daytime.tcp=="enable" then
      inetdConf = inetdConf..
         ("daytime    stream  tcp     nowait  root    internal\n")
   end
   if conf.daytime and conf.daytime.udp=="enable" then
      inetdConf = inetdConf..
         ("daytime    dgram   udp     wait    root    internal\n")
   end

   if conf.chargen and conf.chargen.tcp=="enable" then
      inetdConf = inetdConf..
         ("chargen    stream  tcp     nowait  root    internal\n")
   end
   if conf.chargen and conf.chargen.udp=="enable" then
      inetdConf = inetdConf..
         ("chargen    dgram   udp     wait    root    internal\n")
   end

   local f = io.open("/etc/inetd.conf", 'w')
   if f then
      f:write(inetdConf)
      f:close()
    os.execute"/etc/init.d/inetd reload >/dev/null 2>&1"
   end
   return true
end

inetdInternalScheme = s.newScheme{
    udp = s.choiceType{_H = "servicesInternalUdp", _A={"enable","disable"}, _D = "disable"},
    tcp = s.choiceType{_H = "servicesInternalTcp", _A={"enable","disable"}, _D = "disable"},
  }

servicesScheme = s.newScheme{
  _Z = restartInetd,
  _H = "services",

  ssh = s.newScheme{
    _H = "servicesSsh",
    _Z = restartSSHDaemon,

    enable = s.booleanType{_H = "servicesSshEnable"},
    ["debug-level"] = s.newScheme{
      _H = "servicesSshDebugLevel",
      _T = "number",
      _A = {0, 3},
      _V = s.integerVal,
      _D = 0,
    },
    port = s.tcpPortType{
      _H = "servicesSshPort",
      _D = 22,
    },
    ["log"] =  s.newScheme{
      _H = "servicesSshLog",
      _X = function(path)
        path = s.clonePath(path, -1)
        return C.getDaemonLog(path)
      end,
      _R = 'user',
    },
    keygen = s.newScheme{
      _H = "servicesSshKeygen",
      _T = "string",
      _EX = { subtype = 'areyousure'},
      _V = s.areyousureVal,
      _A = {"\nNew RSA snd DSA host keys will be generated.\n"..
      "It takes a lot of time.\n"},
      _X = function(path, sch, val, sesNum)
        local com
        local yes = val == 'yes'
        if yes then
          com = {'/sbin/keygen-ssh'}
        else
          com = {'echo', 'RSA snd DSA host keys NOT generated.'}
        end
        local myLog = ''
        C.restartDaemon(path, com,
          {sesNum = sesNum, noTimestamp = true, noRestart = true}
        ) 
      end,
      _XI = true,
    },
  },

  http = s.newScheme{
     _H = "servicesHttp",
     _Z = restartHttpDaemon,
     _N = function(sch, conf)
             if (C.getConfOrDefault({"services", "http","https"}) == "enable") then
                return {}
             else
                return { ["renew-cert"] = 1 }
             end
          end,
     enable = s.booleanType{_H = "servicesHttpEnable", _D = true, _DEP = 10},
     --[[
     port = s.tcpPortType{
        _H = "servicesHttpPort",
        _D = 80,
     },
     home = s.stringType{
        _H = "servicesHttpHome",
        _D = "/usr/share/www/"
     },
     --  ]]
     ["log"] =  s.newScheme{
        _H = "servicesHttpLog",
        _X = function(path)
                path = s.clonePath(path, -1)
                return C.getDaemonLog(path)
             end,
        _R = 'user',
     },
     ["https"] = s.choiceType{
        _H = "servicesHttpHTTPS",
        _A = {
           {"enable", ""},
           {"disable", ""},
        },
        _D = "disable",
        _DEP = 10,
     },
     ["renew-cert"] = s.newScheme{
        _H = "servicesHttpRenewCert",
        _X = function(path,  sch, conf, sesNum)
                local si = C.getSessionInfo(sesNum)
           --     if si.rights=="admin" then
                   C.log_warn(s.ser(si))
                   if si.EXTRA["https-renew-cert-warn"] or si.mode ~= "html" then
                      os.execute("rm -f /etc/stunnel/https_auto.cert; rm -f /etc/stunnel/https_auto.key")
                      return s.getManual({ _H = "servicesHttpRenewCertWarnAfter"}, si.locale)
                   else
                      si.EXTRA["https-renew-cert-warn"] = true
                      return s.getManual({ _H = "servicesHttpRenewCertWarnBefore"}, si.locale)
                   end
              --  else
              --     return nil,  s.getManual({ _H = "permissionsDenied"}, si.locale)
               -- end
             end,
     },
  },

  telnet = s.newScheme{
    _H = "servicesTelnet",
    --_Z = restartInetd,

    enable = s.booleanType{_H = "servicesTelnetEnable", _D = true},
    port = s.tcpPortType{
      _H = "servicesTelnetPort",
      _D = 23,
    },
  },
  echo = inetdInternalScheme{
    _H = "servicesEcho",
  },
  discard = inetdInternalScheme{
    _H = "servicesDiscard",
  },
  chargen = inetdInternalScheme{
    _H = "servicesChargen",
  },
  daytime = inetdInternalScheme{
    _H = "servicesDaytime",
  },

  dhcp = dhcp.DhcpScheme,

  dns = dhcp.DnsServerScheme,

  netping = netping.NetPingScheme,
  daemons = daemons.Daemons,
}
