module(..., package.seeall)
local s = require'scheme'
local ppp = require'nsgpppconf'
local usb = require'nsgusbconf'

function getNsgsmsCom(ttyDev, smsconf)
  local com = {'nsgsms', ttyDev,
    '-n', tostring(smsconf['number-of-sms']),
    '-i', tostring(smsconf['inquiry-time']) or 60,
    '-c', tostring(smsconf['control-tcp-port']),
  }
  local dl = smsconf["debug-level"]
  if dl > 0 then
    com[#com+1] = '-l'
    com[#com+1] = '/var/log/sms.log' 
    com[#com+1] = '--timestamp'
    if dl > 1 then
      com[#com+1] = '-'..('v'):rep(dl - 1)
    end
  end
  for _, uphone in ipairs(smsconf['user-phone'] or {}) do
    com[#com+1] = '-u'
    com[#com+1] = tostring(uphone)
  end
  return com, 2 -- ttyDev parameter position
end

function restartSMSDaemon(path, sch, conf)
  local conf = s.addDefault(path, sch, conf, true) --clone conf with defaults
  local smsconf = conf['sms-handler']  
  local ttyDev = usb.port2tty[path[#path]]
  local sim = smsconf.sim == 'aux' and usb.auxSimNumber or usb.mainSimNumber

  local com, ttyParPosition = getNsgsmsCom(ttyDev, smsconf)
  C.restartDaemon(path, com, 
    {
      noTimestamp = true,
      atInit = function(oper)
        local path = oper.options.extra.path
        oper.log:write("Port initialization, wait... ")
        usb.setGPIO(path)
        usb.setSim(path, sim)
      end,
      atStart = function(oper) 
        local path = oper.options.extra.path
        usb.togglePower(path)
        C.threadSleep(5)
        for i = 1, 30 do
          if usb.probeUSBDeviceScheme(nil, path) then
            oper.options.extra.realtty(oper, usb.port2tty[path[#path]])
            break
          end
          C.threadSleep(1)
        end
         oper.log:write("OK\r\n")
      end,
      atExit = C.increasingSleep,
      extra = {
        path = path,
        realtty = function(oper, realtty)
          oper.com[ttyParPosition] = realtty
        end
      },
    }
  ) 

end

smsScheme = s.newScheme{
  _H = "sms",

  ["debug-level"] = s.newScheme{
    _H = "debugLevel",
    _T = "number",
    _A = {0, 4},
    _V = s.integerVal,
    _D = 0,
  },

  ["control-tcp-port"] = s.tcpPortType{
    _H = "smsControltcpport",
    _D = 50000,
  },

  ["inquiry-time"] = function(_, path)
    local enc = C.getConfOrDefault(s.clonePath(path, -2, "encapsulation"))
    return enc == 'ppp' and s.newScheme{
      _H = "smsInquirytime",
      _T = "number",
      _A = {10, 86400},
      _V = s.integerVal,
      _D = 60,
    }
  end,

  ["number-of-sms"] = s.newScheme{
    _H = "smsNumberofsms",
    _T = "number",
    _A = {1, 8},
    _V = s.integerVal,
    _D = 2,
  },

  ["ppp-cooperation"] = function(_, path)
    local enc = C.getConfOrDefault(s.clonePath(path, -2, "encapsulation"))
    return enc == 'ppp' and s.booleanType{
      _H = "smsPppcooperation",
      _D = false,
      _DEP = 2,
    }
  end,

  ["user-phone"] = s.newScheme{
    _H = "smsUserphoneList",
    _LIST = s.newScheme{
      _H = "smsUserphone",
      _T = "string",
      _V = s.anyVal,
    },
  },

  sim = function(_, path)
    return ppp.isDualSimModule(path[#path - 2]) and
    s.newScheme{
      _H = "portRawAccessSim",
      _T = 'choice',
      _A = {"main", "aux"},
      _V = s.choiceVal,
      _D = "main",
    }
  end,
}
