module(..., package.seeall)
local s = require'scheme'

local clonePath = s.clonePath
local function getParent(path, sesNum)
  local path = clonePath(path, -1)
  local sch = C.getSchemeByPath(path)
  local conf = C.getConfByPath(path) or {}
  local locConf = C.getConfByPath(path, sesNum) or {}
  for k, v in pairs(locConf) do
    conf[k] = v
  end
  conf = s.addDefault(path, sch, conf, true) --clone conf with defaults
  return sch, conf
end

local launchPing = function(path,sch, conf, sesNum)
  local sch, conf = getParent(path, sesNum)
  local com = {
    "ping",
    conf.host,
    '-c', tostring(conf.count),
    '-s', tostring(conf.size),
  }
  if conf["source-ip"] then
    com[#com+1] = '-I'
    com[#com+1] = conf["source-ip"]
  end
  C.restartDaemon(path, com,
    {sesNum = sesNum, noTimestamp = true, noRestart = true}) 
end

local launchTraceroute = function(path,sch, conf, sesNum)
  local sch, conf = getParent(path, sesNum)
  local com = {
    "traceroute",
    conf.host,
    '-w', tostring(conf.wait),
  }
  if conf["source-ip"] then
    com[#com+1] = '-s'
    com[#com+1] = conf["source-ip"]
  end
  C.restartDaemon(path, com, 
    {sesNum = sesNum, noTimestamp = true, noRestart = true}) 
end

local launchTelnet = function(path, sch, conf, sesNum)
  local sch, conf = getParent(path, sesNum)
  local com = {
    "telnet",
    conf.host,
    tostring(conf.port),
  }
  if conf["escapechar"] then
    com[#com+1] = '-e'
    com[#com+1] = conf["escapechar"]
  end
  C.restartDaemon(path, com, 
    {sesNum = sesNum, noTimestamp = true, noRestart = true}) 
end

local function getUserName(sesNum)
  local si = C.getSessionInfo(sesNum)
  if si and si.user then
    return si.user..'@'
  else
    return ''
  end
end
local launchSSH = function(path, sch, conf, sesNum)
  local sch, conf = getParent(path, sesNum)
  local com = {
    "ssh",
    (conf.user and conf.user ..'@' or getUserName(sesNum))..conf.host,
    '-p', tostring(conf.port),
  }
  if conf["escapechar"] then
    com[#com+1] = '-e'
    com[#com+1] = conf["escapechar"]
  end
  C.restartDaemon(path, com, 
    {sesNum = sesNum, noTimestamp = true, noRestart = true}) 
end

local launchTcpdump = function(path,sch, conf, sesNum)
  local sch, conf = getParent(path, sesNum)
  local com = {
    "tcpdump",
    "-x",
  }
  if conf["write-to-file"] and not conf["write-to-file"] ~= "no" then
    com[#com+1] = '-w'
    com[#com+1] = conf["write-to-file"]
  end
  if conf.interface and conf.interface:match'^%s*$' then
    com[#com+1] = '-i'
    com[#com+1] = conf.interface
  end
  if conf.expression then
    com[#com+1] = conf.expression
  end
  C.restartDaemon(path, com, 
    {sesNum = sesNum, noTimestamp = true, noRestart = true}) 
end

local launchScheme = s.newScheme{
  _H = "toolsLaunch",
  _XI = true,
  _P = 100,
}

toolsScheme = s.newScheme{
  _H = "tools",


  ping = s.newScheme{
    _H = "toolsPing",
    launch = launchScheme{_X = launchPing, _R = 'user'},

    host = s.ipAddressType{
      _H = "toolsPingHost",
      _D = "127.0.0.1",
      _LOCAL = true,
      _R = 'user'
    },
    ["source-ip"] = s.ipAddressType{
      _H = "toolsPingSourceIP",
      _LOCAL = true,
      _R = 'user'
    },
    count = s.integerType{
      _H = "toolsPingCount",
      _A = {0, math.huge},
      _LOCAL = true,
      _R = 'user',
      _D = 0,
    },
    size = s.integerType{
      _H = "toolsPingSize",
      _A = {0, 65507},
      _D = 56,
      _LOCAL = true,
      _R = 'user',
    },
  },

  traceroute = s.newScheme{
    _H = "toolsTraceroute",
    launch = launchScheme{_X = launchTraceroute, _R = 'user'},

    host = s.ipAddressType{
      _H = "toolsTracerouteHost",
      _D = "127.0.0.1",
      _LOCAL = true,
      _R = 'user',
    },
    ["source-ip"] = s.ipAddressType{
      _H = "toolsTracerouteSourceIP",
      _LOCAL = true,
      _R = 'user',
    },
    wait = s.integerType{
      _H = "toolsTracerouteWait",
      _A = {2, 86400},
      _D = 3,
      _LOCAL = true,
      _R = 'user',
    },
  },

  telnet = s.newScheme{
    _H = "toolsTelnet",
    launch = launchScheme{_X = launchTelnet, _R = 'user'},

    host = s.ipAddressType{
      _H = "toolsTelnetHost",
      _D = "127.0.0.1",
      _LOCAL = true,
      _R = 'user',
    },
    port = s.tcpPortType{
      _H = "toolsTelnetPort",
      _D = 23,
      _LOCAL = true,
      _R = 'user',
    },
    escapechar = s.stringType{
      _H = "toolsTelnetEscapechar",
      _LOCAL = true,
      _R = 'user',
    }
  },

  ssh = s.newScheme{
    _H = "toolsSSH",
    launch = launchScheme{_X = launchSSH, _R = 'user'},

    host = s.ipAddressType{
      _H = "toolsSSHHost",
      _D = "127.0.0.1",
      _LOCAL = true,
      _R = 'user',
    },
    port = s.tcpPortType{
      _H = "toolsSSHPort",
      _D = 22,
      _LOCAL = true,
      _R = 'user',
    },
    user = s.stringType{
      _H = "toolsSSHUser",
      _LOCAL = true,
      _R = 'user',
    },
    escapechar = s.stringType{
      _H = "toolsSSHEscapechar",
      _LOCAL = true,
      _R = 'user',
    },
  },

  tcpdump = s.newScheme{
    _H = "toolsTcpdump",
    launch = launchScheme{_X = launchTcpdump},

    interface = s.stringType{
      _H = "toolsTcpdumpInterface",
      _LOCAL = true,
    },
    expression = s.stringType{
      _H = "toolsTcpdumpExpression",
      _LOCAL = true,
    },
    ["write-to-file"] = s.newScheme{
      _H = "toolsTcpdumpWritetofile",
      _T = 'choice',
      _A = {"no", "/tmp/tcpdump.cap"},
      _V = s.choiceVal,
    },
  },

}
