module(..., package.seeall)
local s = require'scheme'
local WiFiAP = require'nsgwifiap'
local WiFiST = require'nsgwifist'
local iface_mod = require'nsgifaceconf'
local usb = require'nsgusbconf'
----------------------------------------

local function restartWiFi(path, sch, conf, prevConf)
   local ApCmdStart = {}
   local conf = s.addDefault(path, sch, conf, true) --clone conf with defaults
   local wlandev = usb.port2netdev[path[#path]]
   if conf.mode == "access-point" then
      ApCmdStart = { 'hostapd', '/etc/hostapd.conf' }
      WiFiAP.createWiFiConfFile(conf, wlandev)
   elseif conf.mode == "station" then
      ApCmdStart = {'/usr/local/sbin/wpa_supplicant', '-Dwext', '-i' .. wlandev, '-c/etc/supplicant.conf' }
      WiFiST.createWiFiClientConfFile(conf, path)
   end
   
   if conf["debug-level"] > 0 then
      ApCmdStart[#ApCmdStart+1] =  '-'..("d"):rep(conf["debug-level"]) 
   end

   if conf["adm-state"] ~= 'down'then
      C.restartDaemon(path, ApCmdStart, {atExit = C.increasingSleep})
      return true
   end
   C.restartDaemon(path)
   return true
end
----------------------------------------
local WiFiGetStatus = s.newScheme {
   _H = "WifiShowStatus",
   _X = function(path)
	   local path = s.clonePath(path, -2)
	   local wlandev = usb.port2netdev[path[#path]]
	   local conf = C.getConfByPath(path) or {}
	   local out = "\n\tPort is administratively "
	   if conf and conf["adm-state"] == "up" then
	      out = out .. "UP"
	   else
	      out = out .. "DOWN"
	   end
	   out = out .. "\n\n\tWireless Interface (" .. tostring(wlandev or "UNKNOWN") .. ")\n\n"
	   if wlandev then
	      out = out .. s.readpopen("iwconfig " .. wlandev .. " 2>&1")
	   end
	   return out
	end,
   _R = 'user',
}
----------------------------------------
local WiFiGetInfo = s.newScheme {
   _H = "WifiInfo",
   _P = 10,
   _R = 'user',
   _X = function(path)
	   local path = s.clonePath(path, -2)
	   local wlandev = usb.port2netdev[path[#path]]
	   if wlandev then
	      local phyname = string.gsub(wlandev, "wlan", "phy")
	      return s.readpopen("iw phy " .. phyname .. " info 2>&1")
	   else
	      return "No WLAN device"
	   end
	end,
}
local WiFiGetInterface = s.newScheme {
   _H = "WifiInterface",
   _P = 10,
   _R = 'user',
   _X = function(path)
	   local path = s.clonePath(path, -2)
	   local wlandev = usb.port2netdev[path[#path]]
	   if wlandev then
	      return s.readpopen("ifconfig " .. wlandev .. " 2>&1")
	   else
	      return("No device")
	   end
	end,
}
local WiFiShow = s.newScheme {

   _H = "WiFiShow",

   ["log"] =  s.newScheme{
      _H = "WifiLog",
      _X = function(path)
	      path = s.clonePath(path, -2)
	      return C.getDaemonLog(path)
	   end,
      _P = 11,
      _R = 'user',
   },  

   capability = WiFiGetInfo,

   status = WiFiGetStatus,

   clients = function(_, path)
		local mode = C.getConfOrDefault(s.clonePath(path, -2, "mode"))
		return mode == 'access-point' and WiFiAP.WiFiGetConnected
	     end,

   interface = WiFiGetInterface, 
   
}
----------------------------------------

WiFiScheme = s.newScheme {
   _H = "Wifi",
   _Z = restartWiFi,
   _N = function(sch, conf)
	   local need = {
	      true,
	      mode = true,
	      show = true,
	      type = true,   -- need for portTemplate
 	      update = true, -- need for portTemplate
	   }
	   if conf and conf.mode then
	      need["debug-level"] = true
	      need["adm-state"] = true
	      need["ifAddress"] = true
	      if conf.mode == "station" then
		 need["scan"] = true
		 need["connection"] = true
	      else
		 need["ssid"] = true
		 need["security"] = true
		 need["detail"] = true
	      end
	   end
	   return need
	end,
   
   mode = s.newScheme {
      _H = "WifiMode",
      _T = 'choice',
      _A = {
	 {"access-point", "Server device which makes the Wireless Infrastructure",},
	 {"station", "Client device of a infrastructure (managed)"},
      },
      _V = s.choiceVal,
      _P = 20,
      _D = "station",
      _DEP = 1,
   },

   ["adm-state"] = s.newScheme{
      _T = "choice",
      _A = {"up", "down"},
      _V = s.choiceVal,
      _H = "portAdmState",
      _D = "down",
      _P = 19,
   },
   
   -- access-point parameters
   ifAddress = iface_mod.ifAddressListScheme,

   ssid = s.stringType{_H = "WifiApSsid",  _P = 18, _D = "", },

   security = WiFiAP.WiFiSecurity,
      
   detail = WiFiAP.WiFiDetail,  

   -- Parameters of station (clients)

   scan = WiFiST.WiFiScan,

   connection = WiFiST.WiFiConnection,
   -- 
   show = WiFiShow,

  ["debug-level"] = s.newScheme{
    _H = "debugLevel",
    _T = "number",
    _A = {0, 2},
    _V = s.integerVal,
    _D = 0,
    _P = 5,
  },

}
