module(..., package.seeall)
local scheme = require"scheme"
local nsg = require"nsg"

local type = type
local tostring = tostring
local strf = string.format


--------------------------------------------------------------------------------
-- Text mode functions
--------------------------------------------------------------------------------

local function getAllKeysStartingWith(path, node, conf, s, res, nodeNAME, need)
  need = need or (node._N and node._N(node, conf) or {})
  res = res or {}
  local iter = node._pairs and node._pairs or pairs
  for k, v in iter(node, path) do
    if type(s) == "string" and type(k) == "string"
    and k:sub(1,1) ~= '_'
    and not res[k]
    and need[k] == need[1]
    and ((nodeNAME or v)(nil, scheme.clonePath(path, nil, k)))
    and s == k:sub(1, #s) then
      res[#res+1] = k; res[k] = true
      if #s == #k then
        res._match = #res
        return res
      end
    elseif s == '' and type(k) == 'number' then
      res[#res+1] = k
    elseif type(s) == 'number' and k == s then
      res[#res+1] = s
      res._match = #res
      return res
    end
  end
  if node._LIST and conf then
    if s == '' then
      for k, _ in ipairs(conf) do res[#res+1] = k end
    elseif type(s) == 'number' and conf[s] ~= nil then
      res[#res+1] = s
      res._match = #res
      return res
    end
  end
  if node._NAME and conf then --TODO
    res = getAllKeysStartingWith(path, conf, nil, s, res, node._NAME, need)
    if not res._match then
      table.sort(res)
    end
  end
  return res
end

local function maxLengMember(t, min)
  local max = min or 8
  for _, v in ipairs(t) do
    local l = #tostring(v)
    if l > max then max = l end
  end
  return max
end

local function utf8len(ustring)
  local _, count = string.gsub(ustring, "[^\128-\193]", "")
  return count
end

local function wrapLine(s, sp, utf)
  local n = 80 - sp
  if utf8len(s) <= n then return s end
  local t = {}
  local a, b = 1, 0
  repeat
    a, b = s:find('[^%s]+', b + 1)
    if a then
      t[#t+1] = {a,b}
    end
  until not a
  local text = {}
  local st, en = 1, 1
  repeat
    if t[en][2] - t[st][1] + 1 > n then
      if en > st then
        text[#text+1] = s:sub(t[st][1], t[en-1][2])
        st = en
      else
        text[#text+1] = s:sub(t[st][1], t[st][1] + n)
        t[st][1] = t[st][1] + n + 1
      end
    else
      en = en + 1
    end
  until en > #t
  text[#text+1] = s:sub(t[st][1], t[en-1][2])
  local spaces = string.rep(' ', sp)
  return table.concat(text,'\n'..spaces)
end

local colourStyleBranch = nsg.colourstyle("blue", "bold")
local colourStyleCommand = nsg.colourstyle("green")
local colourStyleParameter = nsg.colourstyle("bold")
local colourStyleParameterNil = nsg.colourstyle()

local function getHelp(path, sch, conf, el, args)
  local out = "\n"
  local max = maxLengMember(el)
  local format = strf("%%-%ds  %%s\n", max)
  for _, v in ipairs(el) do
    local childSch = scheme.getChildScheme(path, sch, v)
    local color = childSch and (childSch._X and colourStyleCommand or 
      (not childSch._T and colourStyleBranch or 
        (C.getConfByPath(scheme.clonePath(path, nil, v),
          childSch._LOCAL and args.sesNum) ~= nil and 
        colourStyleParameter or colourStyleParameterNil)
      )
    )

    --out = out..strf(format, nsg.colourstring(color, tostring(v)),
    --wrapLine(scheme.getParameterHelp(path, sch, v, args.locale), max + 2, args.locale))

    out = out..nsg.colourstring(color, tostring(v))..
    (' '):rep(max + 2 - #tostring(v))..
    wrapLine(scheme.getParameterHelp(path, sch, v, args.locale), max + 2, args.locale)..
    '\n'


  end
  local need = sch._N and sch._N(sch, conf) or {}
  if sch._NAME and sch._NAME(nil, scheme.clonePath(path, nil, '_NAME')) and 
  need._NAME == need[1] then
    out = out..strf(format, "_new",
      scheme.getParameterHelp(path, sch, "_NAME", args.locale))
  end
  if sch._LIST and need._LIST == need[1] then
    out = out..strf(format, "_insert",
      scheme.getParameterHelp(path, sch, "_LIST", args.locale))
  end
  return out.."\n"
end


local function tostr(s)
  local out
  if type(s) == "string" then
    out = strf('%q', s):gsub('\\\n', '\\n')
  elseif type(s) == 'table' then
    out = scheme.ser(s)
  else
    out = tostring(s)
  end
  return out
end

local getCommonPrefix do
  local function gcp(s1, s2)
    if s1 and s2 then
      local l = math.min(#s1, #s2)
      for i = 1, l do
        if s1:sub(1,i) ~= s2:sub(1, i) then
          return s1:sub(1, i - 1)
        end
      end
      return s1:sub(1, l)
    else
      return s1 or s2
    end
  end
  function getCommonPrefix(list)
    local pr
    for _, v in ipairs(list) do
      if type(v) ~= 'string' then
        return nil
      end
      pr = gcp(pr, v)
    end
    return (pr and pr ~= '') and pr
  end
end

local function getNextNode(args)
  local s = args.val or ''
  local path = args.path
  local node = args.sch
  local conf = args.conf
  local t = getAllKeysStartingWith(path, node, conf, s)
  if s == '' or (#t > 1 and not t._match) then
    return nil, getHelp(path, node, conf, t, args), getCommonPrefix(t)
  elseif #t == 0 then
    return nil, 'Invalid command'
  else
    local com = t[t._match] or t[1]
    local pth = scheme.clonePath(path, nil, com) -- if user scheme, pth will be nil
    local nextNode = scheme.getSubScheme(pth or com, node)

    if nextNode._X then
      if not nextNode._T then
        args.sch = nextNode
        if nextNode._LINK then
          args.path = nextNode._LINK
        else
          args.path[#args.path+1] = com
        end
        args.localPath[#args.localPath+1] = com
        args.conf = nil
        return executeTask(args)
      elseif (nextNode._EX and (nextNode._EX.subtype == 'areyousure')) then
        return com, true, (nextNode._A[1] or '').."Are you sure[yes/no]?"
      end
    end

    return com, nextNode._T
  end
  return nil
end

function setValue(args)
  if not args.sch._T then
    args.conf = C.getConfByPath(args.path, args.sch._LOCAL and args.sesNum)
    return getNextNode(args)
  end

  local res, err, val = args.setValueCommon(args)
  if res then
    return true, '= '..tostr(val), val
  else
    local conf = C.getConfByPath(args.path, args.sch._LOCAL and args.sesNum)
    return false,
    res == nil and strf(
      "\nGot %s, expected %s\ncurr = %s\ndflt = %s\n\n",
      tostr(args.val), (err or "something"), tostr(conf), tostr(args.sch._D))
    or (err or ""),
    conf
  end
end

function executeTask(args)
  local res, text, val, xi = args.executeTaskCommon(args)
  if res then
    local path = args.path
    local _t = args.sch._T
    if not _t then val = path[#path] end
    return _t and true or false, text, val, xi and args.localPath
  else
    local conf = args.conf
    return false,
    res == nil and strf(
      "\nGot %s, expected %s\ncurr = %s\ndflt = %s\n\n",
      tostr(args.val), (text or "something"), tostr(conf), tostr(args.sch._D)) 
    or (text or ''), 
    conf
  end
end

function removeKey(args)
  return args.funcCommon(args)
end

function newKey(args)
  local res, err, val = args.funcCommon(args)
  if not res then
    if res == nil then
      return false,
      strf("\nGot %s, expected %s\n\n",
        tostr(args.val), (err or "something"))
    else
      return false, err
    end
  else
    return true, err, val
  end
end
insertKey = newKey

local function stripTable(t)
  local empty = true
  for k, v in pairs(t) do
    if type(v) ~= 'table' or not stripTable(v) then
      empty = false
    else
      t[k] = nil
    end
  end
  return empty
end

function getConf(args)
  local key, val = {}, {}
  local path = args.path
  local sch = args.sch
  local conf = args.conf
  local full = args.par and args.par[1]
  local localglobal = args.localglobal or 'onlyglobal'
  local onlyValid = not args.allConfig
  local rawMode = args.outputMode == 'raw'
  local emptyVal = {true}
  if full then
    if sch._T then
      if conf == nil and sch._D and (
        localglobal == 'onlylocal' and sch._LOCAL or 
        localglobal == 'onlyglobal' and not sch._LOCAL
      ) then
        conf = sch._D
      end
    else
      conf = scheme.addDefault(path, sch, conf, true, localglobal, emptyVal)
    end
  end
  if type(conf) == 'table' then stripTable(conf) end
  if rawMode then
    if onlyValid then
      scheme.confValidate(path, sch, conf, true)
    end
    return scheme.ser(conf, 0)
  end
  for p, v in scheme.confPairs(path, sch, conf, onlyValid, true) do
    if v == emptyVal then v = nil end
    key[#key+1] = scheme.clonePath(p)
    val[#val+1] = type(v) == 'table' and '' or '= '..scheme.tostr(v)
  end
  local prevPath = {}
  for i = 1, #key do
    local newPath = {}
    for k, v in ipairs(prevPath) do
      if v == key[i][k] then
        newPath[k] = ': '
      else
        break
      end
    end
    for k = #newPath + 1, #key[i] do
      local keyName = key[i][k]
      if type(keyName) ~= 'string' or 
      keyName:match'[" =\n]' or keyName:match"^[:{]" or keyName:match'^%d*$' then
        keyName = scheme.tostr(keyName)
      end
      newPath[k] = keyName
    end
    prevPath = key[i]
    key[i] = table.concat(newPath)
  end
  local maxlen = maxLengMember(key, 0)
  local out = ''
  local format = strf("%%-%ds %%s\n", maxlen)
  for i = 1, #key do
    out = out..strf(format, key[i], val[i])
  end
  return out
  --return scheme.ser(args.conf, unpack(args.par))
end

function getLocalConf(args)
  args.localglobal = 'onlylocal'
  return getConf(args)
end

function getManual(args)
  return args.man
end
