#!/usr/bin/lua

gp_on = function()end
gp_off = gp_on
--require'strict'
--exports{'nsg', 'scheme'}

package.path = "/etc/nsg/share/lua/5.1/?.lua;"..package.path
package.cpath = "/etc/nsg/lib/lua/5.1/?.so;"..package.cpath

local nsg = require"nsg"
local help = require'nsgconfhelp'

local optsDescr = {
  ("%s [OPTION ...]"):format(arg[0]),
  "",
  "OPTIONS:",
  help = {0, "display this help and exit"},
  h = 'help',
  d = {0, "daemon mode"},
  conf = {1, 'configuration file', 'FILE'},
  c = 'conf',
}
local arg = nsg.getopt({...}, optsDescr)
if arg.help then
  print(nsg.usage(optsDescr))
  return
end

local _ = nil
if arg.d then
  local daemon = daemon or nsg.daemon
  if daemon then
    local res, err = daemon()
    if not res then
      print(err)
    else
      _ = true
    end
  else
    print"no daemon function"
  end
end

local logger, loggerFile
if _ then
  logger = nsg.logging.file("/var/log/nsgconfd.log", '%F %T')
  loggerFile = true
else
  logger = nsg.logging.console('%F %T')
end

local logLevelTab = {
  ["debug"] = nsg.logging.DEBUG,
  ["info"] = nsg.logging.INFO,
  ["warn"] = nsg.logging.WARN,
  ["error"] = nsg.logging.ERROR,
  ["fatal"] = nsg.logging.FATAL,
}
local function setLogLevel(level)
  logger:setLevel(logLevelTab[level or "info"] or nsg.logging.INFO)
end
setLogLevel"info"

local function setLogMaxFilesize(size)
  if loggerFile then
    logger:append(nil, nil, size)
  end
end

local strf = string.format
local function log_debug(...)
  logger:debug(strf(...))
end

local function log_info(...)
  logger:info(strf(...))
end
local log = log_info

local function log_warn(...)
  logger:warn(strf(...))
end

local function log_error(...)
  logger:error(strf(...))
end

local function log_fatal(...)
  logger:fatal(strf(...))
end

gp_off()
--require"classlib"
require"copas"
gp_on()

local type = type
local tostring = tostring
local strf = string.format
local tinsert, concat = table.insert, table.concat
local time, date = os.time, os.date

local countedPcall do
  local function countHook(...)
    local info = debug.getinfo(2)
    if info then
      io.write(("May be deadlock in %s:%d"):
        format(info.short_src, info.currentline))
    end
    error("May be deadlock in function")
  end
  function countedPcall(func, ...)
    debug.sethook(countHook, '', 10*10^6)
    --local res =  {pcall(func, ...)}

    local args = {...}
    local nargs = select("#", ...)
    local res = {xpcall(function() return func(unpack(args, 1,nargs))end, debug.traceback)}

    debug.sethook()
    return unpack(res, 1, table.maxn(res))
  end
end

local f, err = loadfile'/etc/nsgSysConfig'
local nsgc = {}
if f then
  local res, config = countedPcall(f)
  if res and type(config) == 'table' then
    nsgc = config
  end
else
  log_warn(err)
end

local packagePath = nsgc.packagePath
if packagePath then
  package.path = packagePath..';'..package.path
end

local scheme = require"scheme"
local s = scheme
--local serialize = require'serialize'
local ser = nsg.serialize
local cmp = nsg.compareObject
local deepcopy = nsg.deepcopy

-- Get board info
local function getArch()
  local f = io.open("/proc/cpuinfo")
  local cpuinfo, arch, mach = "", nil
  if f then
    cpuinfo = f:read('*a')
    f:close()
  end
  if cpuinfo:match"cpu%s+model%s*:%s*MIPS" then
    arch = "nsg600"
  elseif cpuinfo:match"Hardware%s*:%s*A[Tt][Mm][Ee][Ll]" then
    arch = "nsg700"
  elseif cpuinfo:match"platform%s*:%s*NSG%-18xx" then
    arch = "nsg1800"
  elseif cpuinfo:match"NSG%-VM" then
    arch = "nsgVM"
  elseif cpuinfo:match"NSG%-1000" then
    arch = "nsg1000"
  else
    return nil
  end

  return {
    arch = arch,
  }

end
BI = getArch()
if not BI then
  local mes = "Can't get arch info"
  log_fatal(mes)
  error(mes)
end

local function setBIparams(bi)
  local biModule = require("nsgconf-"..(bi.arch or 'UNKNOWN'))
  if biModule and biModule.set then
    biModule.set(bi)
  else
    error("Can't get board info")
  end
end
setBIparams(BI)
log_info("%s(%s), SN=%s, MAC=%s", 
  tostring(BI.mach), 
  tostring(BI.ports), 
  tostring(BI.serial), 
  tostring(BI.ethaddr)
)


-- set board dependant parameters

-- Find mainScheme and config
local mainScheme

local nsgConfig = {
  schemeFile = arg[1] or nsgc.nsgSchemeFile
  or "/usr/share/lua/5.1/nsgconfscheme.lua",
  configFile = arg.conf or nsgc.nsgConfigFile or "/etc/nsgconfig",
}

if type(nsgConfig.schemeFile) ~= 'string' then
  error("Parameter schemeFile in /etc/nsgConfig must be a string")
end

--mainScheme = assert(loadfile(nsgConfig.schemeFile))()
local res
local f, e = loadfile(nsgConfig.schemeFile)
if not f then
  log_fatal("Error loading configuration scheme: %s", e)
  return
end
res, mainScheme = pcall(f)
if not res then
  log_fatal("Error loading configuration scheme: %s", mainScheme)
  return
end

if type(nsgConfig.configFile) ~= 'string' then
  error("Parameter configFile in /etc/nsgConfig must be a string")
end

local ox = require'posix'

local function loadConfig()
  local f, e = loadfile(nsgConfig.configFile)
  if not f then
    log_error("Error loading config file: %s", e)
  else
    local res
    res, ROOT_CONF = countedPcall(f)
    if not res then
      log_error("Error in config file: %s\n", ROOT_CONF)
    end
  end
  if not ROOT_CONF then
    ROOT_CONF = {}
  end
end

loadConfig()
local prevConf = {} -- previos config for apply function

-- common locals
local configChanged
local S -- Session table
local users = {}
--------------------------------------------------------------------------------
-- Auxiliary functions

local function sockRead(self, pattern, prefix)
  pattern = pattern or 2048
  local data, emsg, partial = self:receive(pattern, prefix)
  if data then
    return data
  end
  if partial and #partial > 0 then
    return partial
  end
  return nil, emsg
end

local MESSAGE_DELIMITER = '\003'

local function sockWrite(self, mes)
  mes = mes:gsub('\\','\\\\'):
  gsub(MESSAGE_DELIMITER, '\\'..MESSAGE_DELIMITER)..MESSAGE_DELIMITER
  --log_debug('reply: %s', #mes <= 80 and mes or 
  --  mes:sub(1, 70).."<...>"..mes:sub(-10)
  --)
  self:send(mes)
end

local function myWrap(skt)
  skt = copas.wrap(skt)
  skt:settimeout(0)
  skt.rcv = sockRead
  skt.snd = sockWrite
  return skt
end

--------------------------------------------------------------------------------
local timer = nsg.timer()

--------------------------------------------------------------------------------
local rightsNameToLevel = {
  user = 1,
  admin = 2,
  root = 3,
}
setmetatable(rightsNameToLevel, {__index = function() return 1 end })


local function toStr(s)
  local out
  if type(s) == "string" then
    out = strf('%q', s)
  elseif type(s) == 'table' then
    out = ser(s)
  else
    out = tostring(s)
  end
  return out
end

local function getConfByPath(path, sesNum)
  local confEnv = sesNum and S[sesNum] or _G
  return scheme.getValByPath(confEnv.ROOT_CONF, path)
end

local function getPrevConfByPath(path)
  return scheme.getValByPath(prevConf, path)
end


local function createUserConf(sch, sesNum)
  if sch._LINK then
    local conf
    if sch._LINK == true then
      conf = {}
      local confNotEmpty
      for k, v in pairs(sch) do
        if type(k) == 'string' and k:sub(1,1) ~= '_' or type(k) == 'number' then
          local c = createUserConf(v, sesNum)
          conf[k] = c
          if c ~= nil then confNotEmpty = true end
        end
      end
      if not confNotEmpty then conf = nil end
    else
      conf = getConfByPath(sch._LINK, sesNum)
    end
    return conf
  end
end

local function getUserConfByPath(ses, path, sesNum)
  local sch = users[ses.user].userSch
  local rootUserConf = createUserConf(sch, sesNum)
  return scheme.getValByPath(rootUserConf, path)
end

local function getUserPrevConfByPath(path)
  return scheme.getValByPath(prevConf, path)
end

local function getSchemeByPath(path)
  local schParent
  local sch = mainScheme
  for k, v in ipairs(path or {}) do
    schParent = sch
    sch = scheme.getSubScheme(scheme.clonePath(path, k), schParent)
    if not sch then break end
  end
  return sch, schParent
end

local function getConfOrDefault(path)
  local cnf = getConfByPath(path)
  if cnf then
    return cnf
  else
    local sch = getSchemeByPath(path)
    return sch and sch._D
  end
end

local function getConfWithDefault(path)
  local cnf = getConfByPath(path)
  local sch = getSchemeByPath(path)
  if sch then
    return scheme.addDefault(path, sch, cnf, true)
  end
end

local function getUserSchemeByPath(ses, path)
  local schParent
  local sch = users[ses.user].userSch
  local link = sch._LINK
  local globalPath = type(link) == 'table' and scheme.clonePath(link) or nil

  local link, edge
  for k, v in ipairs(path or {}) do
    schParent = sch
    local operPath
    if globalPath then
      operPath = scheme.clonePath(globalPath, nil, v)
    else
      operPath = scheme.clonePath(path, k)
    end
    sch = scheme.getSubScheme(operPath, schParent)
    if not sch then 
      globalPath = nil
      break 
    end
    link = sch._LINK
    if link then
      edge = type(link) == 'table'
      globalPath = edge and scheme.clonePath(link) or nil
    else
      globalPath[#globalPath+1] = v
      edge = false
    end
  end
  if globalPath and edge then
    -- We are on an edge of user scheme. Get parent from global scheme.
    local parentPath = scheme.clonePath(globalPath, -1)
    schParent = getSchemeByPath(parentPath)
  end
  return globalPath, sch, schParent
end


local function setConfigValue(path, val, sesNum)
  local confEnv = sesNum and S[sesNum] or _G
  configChanged = sesNum == nil
  if #path == 0 then
    confEnv.ROOT_CONF = val or {}
  else
    scheme.addValToTable(confEnv.ROOT_CONF, path, val)
  end
end

local setOption, getOption do
  local options = {}
  function setOption(path, option, value)
    path = scheme.clonePath(path, nil, "_OPTIONS", option)
    scheme.addValToTable(options, path, value)
  end
  function getOption(path, option)
    path = scheme.clonePath(path, nil, "_OPTIONS", option)
    return scheme.getValByPath(options, path)
  end
end

local function badPathError(path)
  return "Bad or not active menu node: "..table.concat(path, '.')
end

local function getVersion()
  local f = io.open("/version", "rb")
  local release = (f and f:read("*a") or "UNKNOWN"):match'^%s*(.-)%s*$'
  local _ = f and f:close()
  f = io.open("/etc/nsg/spversion", "rb")
  local sp = f and f:read("*a")
  _ = f and f:close()
  sp = sp and "-sp-"..sp or ""
  return release..sp
end
--------------------------------------------------------------------------------

-- Table of session
S = setmetatable({}, {
    __index = function()
      return setmetatable({},{
          __index = function()
            return function()
              return false, 'Session is closed'
            end
      end })
end })

-- define class Session
local Session = {}
local AdminSession = {}
local RootSession = {}

local function newSession(num, rights)
  local self = setmetatable(
    {},
    {
      __index = function(self, func)
        return function()
          if AdminSession[func] or RootSession[func] then
            return false, "Permissions denied\n"
          end
          return false, "Bad request\n"
        end
      end
    }
  )
  self.num = num
  for k, v in  pairs(Session) do
    self[k] = v
  end
  if rights == 'admin' or rights == 'root' then
    for k, v in  pairs(AdminSession) do
      self[k] = v
    end
    if rights == 'root' then
      for k, v in  pairs(RootSession) do
        self[k] = v
      end
    end
  end
  return self
end

-- all interface function must return (res) or (false, error)
local killAllLocalDaemon
function Session:closeSession()
  local n = self.num
  killAllLocalDaemon(n)

  S[n] = nil
  S[0].numberOfOpenSessions = S[0].numberOfOpenSessions - 1

  if S[0].ROOT_SESSION == n then
    S[0].ROOT_SESSION = nil
  end
  if self.rights == 'admin' then
    loadConfig() -- undo all changes
  end
  configChanged = nil
  return'OK'
end

local function getConfOrLocalConf(self, sesNum, path, ...)
  local globalPath, node, nodeParent = getUserSchemeByPath(self, path)
  if not node then
    return false, badPathError(path)
  end
  local cnf, cnfParent = getUserConfByPath(self, path, sesNum)
  local method = sesNum and 'getLocalConf' or 'getConf'
  return node._methods[self.mode][method]{
    path = globalPath,
    virtPath = path,
    sch = node,
    schParent = nodeParent,
    conf = cnf,
    confParent = cnfParent,
    locale = self.locale,
    outputMode = self.outputMode,
    par = {...},
    getConfCommon =  node._methods["text"][method]
  }
end

function Session:getConf(path, ...)
  return getConfOrLocalConf(self, nil, path, ...)
end

function Session:getLocalConf(path, ...)
  return getConfOrLocalConf(self, self.num, path, ...)
end

function Session:setLocale(locale)
  if not locale then
    return self.locale
  else
    self.locale = tostring(locale)
  end
end

function Session:setOutputMode(mode)
  if not mode then
    return self.outputMode
  else
    self.outputMode = tostring(mode)
  end
end

function Session:setMode(mode)
  if not mode then
    return self.mode
  else
    self.mode = tostring(mode)
  end
end

function Session:loadDictionary()
  return  mainScheme._methods[self.mode].loadDictionary()
end

function Session:getHTML(path)  
  local globalPath, sch = getUserSchemeByPath(self, path)
  return  mainScheme._methods[self.mode].getHTML{
    sch = sch,
  }
end

function Session:getManual(path)
  local globalPath, sch = getUserSchemeByPath(self, path)
  if not sch then
    return false, "badPathError(path)"
  end
  local man =  scheme.getManual(sch, self.locale)
  return sch._methods[self.mode].getManual{
    man = man,
  }
end

function Session:validateValue(path, val, dflt)
  local globalPath, sch = getUserSchemeByPath(self, path)
  if not sch then
    return false, badPathError(path)
  end
  return scheme.confValidate(globalPath, sch, val, dflt)
end

local function validateConfCommon(self, path, dflt)
  local globalPath, sch = getUserSchemeByPath(self, path)
  if not sch then
    return false, badPathError(path)
  end
  local cnf = getUserConfByPath(self, path)
  local ret, err = scheme.confValidate(globalPath, sch, cnf, dflt)
  return ret, sch._methods.text.getConf{
    path = path,
    sch = sch,
    conf = err,
    allConfig = true,
    outputMode = self.outputMode,
  }
end

function Session:validateConf(path)
  return validateConfCommon(self, path)
end

function AdminSession:validateAndRepairConf(path)
  return validateConfCommon(self, path, true)
end


local setValueCommon, executeTaskCommon 

do
  local setval = function(args, node, val)
    local env = node._LOCAL and args.sesNum
    setConfigValue(args.path, val, env)
    return true, nil, val
  end

  local exectask = function(args, node, val)
    return true, scheme.exec(args.path, node, val, args.sesNum), val, node._XI
  end

  local function setValueOrExec(args, node, func)
    local s, err = true
    local val = args.val
    if node._T and node._V then
      s, err = node._V(val, unpack(node._A or {}))
      val = s
    end
    if s ~= nil then
       if args.rightsLevel >= rightsNameToLevel[node._R or 'admin'] then
          return func(args, node, val)
       else
          local _H="permissionsDenied"
          return false, scheme.getHelpString(help[_H], args.locale, _H)
       end
    else
      return nil, err
    end
  end

  ---
  -- @param args table of options
  -- options:
  -- path - path to node
  -- sch  - scheme table
  -- conf - configuration table
  -- val  - parameter value
  -- 
  --
  -- @return true or false or nil, error message, accepted value
  -- nil   - validate error
  -- false - other error


  function setValueCommon(args)
    local node = args.sch
    if node._X or not node._T then
      return false, "operation not permitted"
    end
    return setValueOrExec(args, node, setval)
  end

  ---
  -- @param args table of options
  -- options:
  -- path - path to node
  -- sch  - scheme table
  -- conf - configuration table
  -- val  - parameter value
  -- 
  --
  -- @return true or false, error message, accepted value, _XI
  -- nil   - validate error
  -- false - other error


  function executeTaskCommon(args)
    local node = args.sch
    if not node._X then
      return false, "operation not permitted"
    end
    return setValueOrExec(args, node, exectask)
  end

end

function Session:setValue(path, val)
  local globalPath, sch, schParent = getUserSchemeByPath(self, path)
  if not sch then
    return false, badPathError(path)
  end

  local conf, confParent = getConfByPath(globalPath)
  return sch._methods[self.mode].setValue{
    path = globalPath,
    sch = sch,
    schParent = schParent,
    conf = conf,
    confParent = confParent,
    val = val,
    rightsLevel = self.rightsLevel,
    sesNum = self.num,
    locale = self.locale,
    setValueCommon = setValueCommon,
    executeTaskCommon = executeTaskCommon,
    localPath = path,
  }
end

function Session:executeTask(path, val)
  local globalPath, sch = getUserSchemeByPath(self, path)
  if not sch then
    return false, badPathError(path)
  end

  return sch._methods[self.mode].executeTask{
    path = globalPath,
    sch = sch,
    conf = getConfByPath(globalPath),
    val = val,
    rightsLevel = self.rightsLevel,
    sesNum = self.num,
    locale = self.locale,
    executeTaskCommon = executeTaskCommon,
    localPath = path,
  }
end

function Session:setValueSmart(path, val)
  local globalPath, node = getUserSchemeByPath(self, path)
  if not node then
    return false, badPathError(path)
  end
  if node._X then
    return self:executeTask(path, val)
  else
    return self:setValue(path, val)
  end
end

function Session:isConfigChanged()
  return false
end

function Session:getVersion()
  return getVersion()
end



function AdminSession:isConfigChanged()
  return configChanged
end

function AdminSession:saveConf()
  local f, err = io.open(nsgConfig.configFile, 'w')
  if f then
    f:write("return", ser(ROOT_CONF))
    f:close()
    --os.execute'savecfg 2>/dev/null' -- TODO
    ox.chmod(nsgConfig.configFile, 'a-rwx,u+wr')
    configChanged = nil
    return 'Configuration saved'
  else
    return false, strf(
    "Can't write configuration to file %s(%s)", nsgConfig.configFile, err)
  end
end

local function removeKeyCommon(args)
  local path = args.path
  local cnf = getConfByPath(path)
  if cnf ~= nil then
    local name = path[#path]
    local parentPath = scheme.clonePath(path, -1)
    local parentSch = getSchemeByPath(parentPath)
    local parentCnf = getConfByPath(parentPath)
    if (
      type(name) == 'number' 
      and parentSch._LIST 
      and name%1==0 -- integer
      and name > 0 and name <= #parentCnf
    ) then
      table.remove(parentCnf, name)
    else
      setConfigValue(path, nil)
    end
  end
  return true
end

local function removeConfigValue(path)
  return removeKeyCommon{path = path}
end

function AdminSession:removeKey(path)
  local globalPath, sch, parSch = getUserSchemeByPath(self, path)
  --  if sch._LINK then
  if not globalPath then
    return false, "Can't remove this node"
  end

  if not sch then
    return false, badPathError(path)
  end

  return sch._methods[self.mode].removeKey{
    path = globalPath,
    sch = sch,
    funcCommon = removeKeyCommon,
  }
end


local function newKeyCommon(args)
  local val = args.val
  local val, err = (args.sch._NAME_TYPE or scheme.isDefaultNameType)(val)
  if not val then
    return false, err
  end
  local path = args.path

  if args.sch and args.sch[val] ~= nil then
    return false, "Node "..val.." exists."
  end
  local cnf = getConfByPath(path)
  if cnf and cnf[val] then
    return false, "Node "..val.." exists, remove it before."
  end

  path[#path+1] = val
  local sch = getSchemeByPath(path) or {}
  setConfigValue(path, not sch._T and {_keep=true} or (sch._D or false))
  configChanged = true
  return true, sch._T, val
end

function AdminSession:newKey(path, val)
  local globalPath, sch = getUserSchemeByPath(self, path)
  if not sch then
    return false, badPathError(path)
  end
  path = globalPath or path
  if not sch._NAME then
    return false, "New element can not be added to this node"
  end

  return sch._methods[self.mode].newKey{
    path = path,
    sch = sch,
    val = val,
    funcCommon = newKeyCommon,
  }
end

local function insertKeyCommon(args)
  local val = args.val
  local path = args.path
  if val == nil then val = 0x7fffffff end
  if type(val) ~= "number" or val <= 0 or val%1~=0  then
    return nil, "integer > 0"
  end
  local cnf = getConfByPath(path)
  path[#path+1] = 1
  local sch = getSchemeByPath(path) or {}
  local insval = not sch._T and {_keep=true} or false
  if cnf then
    if val > #cnf + 1 then val = #cnf + 1 end
    tinsert(cnf, val, insval)
  else
    setConfigValue(path, insval)
    val = 1
  end
  configChanged = true
  return true, sch._T, val
end

function AdminSession:insertKey(path, val)
  local globalPath, sch = getUserSchemeByPath(self, path)
  if not sch then
    return false, badPathError(path)
  end
  path = globalPath or path
  if not sch._LIST then
    return false, "New element can not be inserted to this node"
  end

  return sch._methods[self.mode].insertKey{
    path = path,
    sch = sch,
    val = val,
    funcCommon = insertKeyCommon,
  }
end

function Session:isApply(path)
  local globalPath, sch = getUserSchemeByPath(self, path)
  if not sch then
    return false, badPathError(path)
  end
  path = globalPath or path
  if sch._Z then
    return true, path
  end
  while #path ~= 0 do
    path[#path] = nil
    if self:isApply(path) then
      break
    end
  end
  return false, path
end

local function applyConfAux(path, first, uncond, sesNum)
  local sch = getSchemeByPath(path) -- global path
  if not sch then
    return false, badPathError(path)
  end
  if sch._Z then
    local cnf = getConfByPath(path)
    local pcnf = getPrevConfByPath(path)
    local res, err = scheme.apply(path, sch, deepcopy(cnf),
      not uncond and pcnf or nil, first, sesNum
    )
    if res == true then
      if #path == 0 then
        prevConf = deepcopy(cnf)
      else
        scheme.addValToTable(prevConf, path, deepcopy(cnf))
      end
      return 'OK'
    else
      log_error("apply error:\n%s", err)
      return false, err
    end
  end
  return false, "Can't apply"
end

function AdminSession:applyConf(path)
   local globalPath, sch = getUserSchemeByPath(self, path)
  return applyConfAux(globalPath or path, nil, nil, self.num)
end

--[[
function AdminSession:addComment(path, comment)
  local globalPath, sch = getUserSchemeByPath(self, path)
  if not sch then
    return false, badPathError(path)
  end
  if sch._T then
    return false, "Comment may be added only to node"
  end
  path[#path+1] = "_COMMENT"
  scheme.addValToTable(ROOT_CONF, path, comment)
end
--]]

function Session:copy(path)
  local globalPath, sch = getUserSchemeByPath(self, path)
  if not globalPath then
    return false, "Can't copy this node"
  end
  local cnf = getConfByPath(globalPath)
  self.clipboard = deepcopy(cnf)
  return true
end

require 'lpeg'
local configLine do
  local C, P, S = lpeg.C, lpeg.P, lpeg.S
  local quoted_field = C('"' * ((P'\\\\' + P'\\"' + P(1) - '"'))^0 * '"')
  local any_field = C((1 - S' =\n\t')^0)
  local field = quoted_field + any_field
  local sp = S' \t'^0
  local prefix = C(P': '^0)
  local record = prefix * field * (sp * '=' * sp * field)^-1 * sp * -1
  function configLine (s)
    return lpeg.match(record, s)
  end
end

local function userConfToTable(userConf)
  if userConf:match"^%s*{%s-\n[^:]" then -- raw mode
    local func, err = loadstring('setfenv(1, {})return '..userConf, "")
    if not func then
      return false, err:gsub("^%b[]:", "", 1)
    end
    local res, confTable = pcall(func)
    if res then
      return confTable
    else
      return false, confTable:gsub("^%b[]:", "", 1)
    end
  else -- user-friendly mode
    local resTable = {}
    local path = {}
    local prevPathIdx = 0
    for line in userConf:gmatch"(.-)\n" do
      if not (line:match"^%s*$" or line:match"^%s*%-%-") then
        local l, k, v = configLine(line)
        if not l then
          return false, "Syntax error in line "..line
        end
        pathIdx = #l/2+1
        path[pathIdx] = (k:sub(1, 1) == '"') and loadstring('return '..k)() or 
        (tonumber(k) or k)

        for i = pathIdx + 1, prevPathIdx do 
          path[i] = nil
        end
        prevPathIdx = pathIdx
        if v then
          local func, err = loadstring('setfenv(1, {})return '..v, "")
          if not func then
            return false, err:gsub("^%b[]:%d+", l..k.." = "..v, 1)
          end
          local res, val = pcall(func)
          if res then
            scheme.addValToTable(resTable, path, val)
          else
            return false, val:gsub("^%b[]:%d+", l..k.." = "..v, 1)
          end
        end
      end
    end
    return resTable
  end
end

function AdminSession:paste(path, userConf, onlyValidate)
  local globalPath, sch = getUserSchemeByPath(self, path)
  if not sch then
    return false, badPathError(path)
  end
  if not globalPath or sch._T then
    return false, "Can't configure this node"
  end

  local userConf, err = userConfToTable(userConf)
  if not userConf then
    return false, err
  end

  -- temporarily set new config
  local oldconfig = getConfByPath(globalPath)
  if #globalPath == 0 then  
    ROOT_CONF = userConf
  else
    setConfigValue(globalPath, userConf)
  end
  configChanged = true

  local res, err = scheme.confValidate(globalPath, sch, userConf)
  if res then
    if onlyValidate then
      if #globalPath == 0 then
        ROOT_CONF = oldconfig
      else
        setConfigValue(globalPath, oldconfig)
      end
      configChanged = false
    end
    return true
  else
    if #globalPath == 0 then
      ROOT_CONF = oldconfig
    else
      setConfigValue(globalPath, oldconfig)
    end
    configChanged = false
    return false, "Can't set config\n"..sch._methods.text.getConf{
      path = globalPath,
      sch = sch,
      conf = err,
      allConfig = true,
      outputMode = self.outputMode,
    }
  end
end


function Session:getPerm()
  return users[self.user].permission, self.rights
end

function Session:isSession()
  return true
end

--------------------------------------------------------------------------------
-- Daemon stuff

local function threadSleep(n, oper)
  local rawSleep = socket.pty('sleep', tostring(n))
  if rawSleep then
    local oldDaemon
    if oper then
      oldDaemon = oper.daemon
      oper.daemon = rawSleep
    end
    rawSleep:settimeout(0)
    local sleep = myWrap(rawSleep)
    local s, err
    repeat
      s, err = sleep:rcv()
    until not s
    sleep.socket:close()
    if oper and oper.daemon == rawSleep then
      oper.daemon = oldDaemon
    end
  else
    log_error"Can't sleep thread"
  end
end

local maxSleepTime = 256
local function increasingSleep(oper) -- may be used only as atExit callback
  if oper.daemon == 'kill' then
    return
  end
  local maxDaemonLifeTime = (oper.options.extra 
    and oper.options.extra.maxDaemonLifeTime or 5
  )
  local sleepTmo = oper.extra.sleepTmo or 1
  local lastExit = oper.extra.lastExit or 0

  if time() - lastExit < maxDaemonLifeTime then -- too few lifetime
    oper.extra.sleepTmo = sleepTmo * (sleepTmo < maxSleepTime and 2 or 1)
    log_warn("Too few daemon lifetime(%s). Sleep for %d sec.", oper.path, sleepTmo)
  else
    oper.extra.sleepTmo = 1
  end
  threadSleep(sleepTmo, oper)
  oper.extra.lastExit = time()
end

local function removeTmpFiles(list)
  for _, fname in ipairs(list) do
    os.remove(fname)
  end
end

local function closeDaemon(oper)
  oper.pipe = nil

  -- remove old tmp files
  if oper.tmpFileList then
    removeTmpFiles(oper.tmpFileList)
    oper.tmpFileList = nil
  end
  oper.stat = {}
end

local function newOperLog()
  return {
    curr = {''},
    prev = {},
    count = 0,

    write = function(self, mes)
      self.count = self.count + #mes
      if self.count > 8192 then -- maximum chars in log
        self.prev = self.curr
        self.curr = {}
        self.count = 0
      end
      tinsert(self.curr, mes)
    end,

    read = function(self, n)
      if n then
        return self.curr[n < 0 and #self.curr + 1 + n or n ]
      else
        return concat(self.prev or {})..concat(self.curr)
      end
    end,

    clean = function(self)
      self.curr = {''}
      self.prev = {}
      self.count = 0
    end,
  }
end

local function initNewDaemon(oper, com, options)
  if com then
    oper.daemon = 'start'
    oper.com = com
    if not oper.log then
      oper.log = newOperLog()
    end
    local options = options or {}
    oper.options = options

    oper.tmpFileList = options.tmpFileList
    oper.noTimestamp = options.noTimestamp
    oper.markStartExit = options.markStartExit
    oper.noRestart = options.noRestart
    oper.usePopen = options.usePopen

    oper.atInit = options.atInit
    oper.atStart = options.atStart
    oper.atRead = options.atRead
    oper.atExit = options.atExit
    oper.atQuit = options.atQuit
    oper.atKill = options.atKill

    oper.extra = {}

  end
end

local function readPipe(f, oper)
  while true do
    if oper.atInit then
      oper.atInit(oper)
    end
    repeat
      if oper.atStart then
        oper.atStart(oper)
      end
      if not oper.com or oper.daemon == 'kill' then break end

      local dname = table.concat(oper.com, ' ')
      log_info("%s: Start daemon %s", oper.path, dname)
      if not oper.noTimestamp or oper.markStartExit then
        oper.log:write('\n----- '..date('%F %T')..' ----- START '..dname..'\n')
      end
      local socktype = oper.usePopen and 'pipe' or 'pty'
      local rawPipe = socket[socktype](unpack(oper.com))
      oper.daemon = rawPipe
      rawPipe:settimeout(0)
      local pipe = myWrap(rawPipe)
      oper.pipe = pipe

      while true do
        local s, err = pipe:rcv()
        local tm = time()
        if s then
          if not oper.noTimestamp and tm ~= oper.logTimestamp then
            oper.log:write('\n----- '..date('%F %T', tm)..' -----\n')
            oper.logTimestamp = tm
          end
          log_debug("%s: %s\n", oper.path, s)
          oper.log:write(s)
          if oper.atRead then
            oper.atRead(oper)
          end

        else
          log_debug("%s: %s\n", oper.path, err)
          --oper.log:write(err..'\n')
          break
        end
      end

      pipe.socket:close()
      oper.exitCode = rawPipe:getstatus()
      log_info("%s: Stop daemon %s", oper.path, dname, ser(oper))
      if not oper.noTimestamp or oper.markStartExit then
        oper.log:write('\n----- '..date('%F %T')..' ----- EXIT '..
          (oper.exitCode.exitstatus or '')..'\n'
        )
      end
      if oper.atExit then
        oper.atExit(oper)
      end
    until oper.noRestart or oper.daemon == 'kill'
    oper.daemon = 'stop'

    if oper.atQuit then
      oper.atQuit(oper)
    end

    closeDaemon(oper)

    if oper.new then
      initNewDaemon(oper, oper.new.com, oper.new.options)
      oper.new = nil
    else
      break
    end
  end
  oper.daemon = nil
end

local function killDaemon(oper)
  if oper.daemon and oper.daemon ~= 'kill' then
    if oper.atKill then
      oper.atKill(oper)
    end
    if type(oper.daemon) ~= 'string' then
      oper.daemon:kill()
    end
    oper.daemon = 'kill'
  end
  if oper.new then
    local list = (oper.new.options or {}).tmpFileList
    if list then
      removeTmpFiles(list)
    end
  end
  oper.new = nil
end

local getOperByPath do
  local oper = {}
  local localOper = {}
  function getOperByPath(path, sesNum)
    local opr
    if sesNum then
      if not localOper[sesNum] then
        localOper[sesNum] = {}
      end
      opr = localOper[sesNum]
    else
      opr = oper
    end
    local rootOper = opr
    for k, v in ipairs(path or {}) do
      opr = opr[v]
      if not opr then
        opr = {_T = true, path = concat(path, '.')}
        scheme.addValToTable(rootOper, path, opr)
        break
      end
    end
    return opr
  end

  local function kald(oper)
    if oper._T then
      killDaemon(oper)
      log_debug('kill daemon %s', oper.path)
    else
      for k, v in pairs(oper) do
        if type(v) == 'table' then
          kald(v)
        end
      end
    end
  end

  function killAllLocalDaemon(sesNum)
    if localOper[sesNum] then
      kald(localOper[sesNum])
    end
  end
end


---Stop and/or start daemon
-- @param path path to node(list of keys such as {k1, k2, k3,...})
-- @param com list {filename, arg1, arg2, ...}
-- @param options table of options
-- options:
--    atInit
--    atStart
--    atRead
--    atExit
--    atQuit
--    atKill       - callback functions
--    tmpFileList  - list of files for removing when close daemon
--    noTimestamp  - don't make timestamps in logs
--    noRestart    - don't restart daemon
--    usePopen     - use popen instead forkpty when start daemon
--    sesNum       - 
--    extra        - table of extra options using in callback function

local function restartDaemon(path, com, options)
  local oper = getOperByPath(path, options and options.sesNum)
  log_debug('restartDaemon %s', ser(path))
  -- kill previous daemon
  killDaemon(oper)

  if not oper.daemon then
    closeDaemon(oper)
    if com then
      initNewDaemon(oper, com, options)
      copas.addthread(readPipe, oper)
    end
  elseif com then
    oper.new = {
      com = com,
      options = options
    }
  end
end

local function getDaemonLog(path, sesNum)
  local oper = getOperByPath(path, sesNum)
  if oper and oper.log then
    return oper.log:read()
  end
end
local function cleanDaemonLog(path, sesNum)
  local oper = getOperByPath(path, sesNum)
  if oper and oper.log then
    return oper.log:clean()
  end
end

local function getDaemonStat(path, sesNum)
  return getOperByPath(path, sesNum).stat
end

function Session:tty(path, com)
  local globalPath, sch = getUserSchemeByPath(self, path)
  path = globalPath or path
  local oper = getOperByPath(path, self.num)
  local working
  if oper.daemon and oper.daemon ~= 'kill' then
    working = true
    if not com then
      restartDaemon(path, nil, {sesNum = self.num}) -- close daemon
    elseif type(com) == 'string' and type(oper.daemon) ~= 'string' then
      -- send com
      oper.daemon:send(com)
    end
  end
  local out
  if oper.log then
    out = oper.log:read()
    oper.log:clean()
  end
  if working then
    return out
  else
    return false, out
  end
end

--------------------------------------------------------------------------------
-- netlink stuff
local neteventCallbacks = {}

local function addNeteventCallback(func)
  local unicKey = {}
  neteventCallbacks[unicKey] = func
  return unicKey
end

local function deleteNeteventCallback(unicKey)
  neteventCallbacks[unicKey] = nil
end

local function updateNeteventCallback(unicKey, func)
  neteventCallbacks[unicKey] = func
  return unicKey
end

local function netevent(_)
  if not socket.netlink then
    log_error("No netlink socket support")
    return
  end
  local nlsock, err = socket.netlink()
  if not nlsock then
    log_error("Can't open netlink socket: %s", err or "")
    return
  end
  nlsock:settimeout(0)
  nlsock = copas.wrap(nlsock)
  while true do
    local res, err = nlsock:receive()
    if not res then
      log_error("Error reading netlink socket: %s", err or "")
      return
    end
    log_debug("netevent: %s", scheme.ser(res))
    for _, msg in ipairs(res) do
      for _, cb in pairs(neteventCallbacks) do
        cb(msg)
      end
    end
  end
end

--------------------------------------------------------------------------------
-- user stuff

users.root = {
  permission = 'rw',
  userSch = getSchemeByPath{}{_LINK = {}},
}
users.nsg = {
  permission = 'rw',
  userSch = getSchemeByPath{}{_LINK = {}},
}

local function createPath(str)
  local path = {}
  str = str:gsub("%.%.", '\1')
  for w in (str..'.'):gmatch"(.-)%." do
    if w ~= '' then
      local nodeName = w:gsub('\1', '.')
      path[#path+1] = tonumber(nodeName) or nodeName
    end
  end
  return path
end

local function createUserScheme(conf)
  local sch
  local link = conf['@link']
  if link then
    local path = createPath(link)
    sch = getSchemeByPath(path)
    if sch then
      sch = sch{_LINK = path, _H = conf['@help']}
    end
  else
    sch = scheme.newScheme{_LINK = true, _H = conf['@help']}
    for k, v in pairs(conf) do
      local firstChar = k:sub(1,1)
      if firstChar ~= '@' and firstChar ~= '_' then
        sch[k] = createUserScheme(v)
      end
    end
  end
  return sch
end

local function excludeNode(sch, excl, idx)
  if not sch then return nil end
  local nextnode = excl[idx]
  if #excl == idx then -- the leaf
    sch[nextnode] = type(sch._LINK) == 'table' and function()end -- because of metatables
    return sch
  else
    if sch._LINK == true then
      sch[nextnode] = excludeNode(sch[nextnode], excl, idx + 1)
      return sch
    else -- the edge element
      local newsch = scheme.newScheme{_LINK = true, _H = sch._H}
      for k, v in sch:_pairs() do
        local path = scheme.clonePath(sch._LINK, nil, k)
        local childsch = getSchemeByPath(path)
        if childsch then
          newsch[k] = childsch{_LINK = path}
        end
      end
      newsch[nextnode] = excludeNode(newsch[nextnode], excl, idx + 1)
      return newsch
    end
  end
end
local function excludeNodesFromUserScheme(sch, excl)
  for i = 1, #excl do
    local path = createPath(excl[i])
    if #path >= 1 then
      sch = excludeNode(sch, path, 1)
    end
  end
  return sch
end

local function addUser(userName, conf)
  local newUser = {
    name = userName,
    permission = conf.permission,
    locale = conf.locale,
    outputMode = conf["output-mode"]
  }
  newUser.userSch = createUserScheme(conf.scheme or {})
  if conf.exclude then
    newUser.userSch = excludeNodesFromUserScheme(newUser.userSch, conf.exclude)
  end

  users[userName] = newUser
end

local function updateRootNsgUser(userName, conf)
  users[userName].locale = conf.locale
  users[userName].outputMode = conf["output-mode"]
end


--------------------------------------------------------------------------------
local lastSessionNumber
local lastSessionNumberFile = '/etc/nsgconfd.sesnum'

local function getLastSessionNumber()
  local f = io.open(lastSessionNumberFile)
  if f then
    lastSessionNumber = tonumber(f:read'*a')
    f:close()
  end
end
getLastSessionNumber()

local function saveLastSessionNumber()
  f = io.open(lastSessionNumberFile, 'w')
  if f then
    f:write(tostring(lastSessionNumber))
    f:close()
    --os.execute'savecfg 2>/dev/null' -- TODO
    ox.chmod(lastSessionNumberFile, 'a-rwx,u+wr')
  end
end

local initApplyError, initMesages
local MAX_NUM_OF_SESSION = 256
local function createNewSession(args)
  if S[0].numberOfOpenSessions > MAX_NUM_OF_SESSION then
    return nil, "Too many sessions"
  end
  local args = args or {}

  if not args.user or not users[args.user] then
    return nil, "Unknown user"
  end

  local maxRights = args.maxRights
  if not maxRights then
    maxRights = 'user'
    if users[args.user].permission == 'rw' then
      maxRights = 'admin'
    end
  end

  local mes
  local rights = 'user'
  local i = S[0].nextSessionNumber
  local rootSes = S[0].ROOT_SESSION
  if (
    rootSes 
    and maxRights == 'admin' 
    and (time() - S[rootSes].lastComTimestamp > 
      (tonumber(getConfOrDefault{'system', 'sessions', 'idle-time'}) or 5*60))
  ) then
    log_info("Sesion %d closed by timeout", rootSes)
    S[rootSes]:closeSession()
    rootSes = nil
  end
  if rootSes then
    mes = "Configuration is locked by other user.\n"
    .."You have READ-ONLY rights"
  else
    rights = maxRights
    if rights == 'admin' then
      S[0].ROOT_SESSION = i
    else
      mes = "You have READ-ONLY rights"
    end
  end

  S[i] = newSession(i, rights)
  S[i].mode = args and args["mode"] or rawget(S[i],"mode") or "text"
  S[i].user = args.user
  S[i].script = args.script or false
  S[i].rights = rights
  S[i].rightsLevel = rightsNameToLevel[rights]
  S[i].lastComTimestamp = time()
  S[i].ROOT_CONF = {}
  S[i].EXTRA = {}
  S[i]:setLocale(users[args.user].locale)
  S[i]:setOutputMode(users[args.user].outputMode)

  lastSessionNumber = i + 1
  S[0].nextSessionNumber = lastSessionNumber
  S[0].numberOfOpenSessions = S[0].numberOfOpenSessions + 1

  saveLastSessionNumber()

  if initApplyError then
    mes = (mes or '').."\n*** Error loading configuration ***\n"..initApplyError..'\n'
  end
  if initMesages then
    mes = (mes or '')..initMesages..'\n'
  end


  return i, mes
end

-- main session
S[0] = {
  nextSessionNumber = lastSessionNumber or 1,
  numberOfOpenSessions = 0,

  init = function(self, args)
    return createNewSession(args)
  end,
}

local function getSessionInfo(sesNum)
  if S[sesNum] and S[sesNum].num == sesNum then
    return S[sesNum]
  end
end

local function showSessions(sesNum)
  local out = {}
  local pattern = "%1s%-6s %-16s %-8s %-5s %-8s"
  out[1] = pattern:format('', "id", "user", "rights", "mode", "inactive")
  out[2] = ('-'):rep(#out[1])
  for i, ses in pairs(S) do
    if ses.num == i then
      out[#out+1] = pattern:format(sesNum == i and '*' or '', tostring(i), 
        ses.user, ses.rights, ses.mode,
        tostring(time() - ses.lastComTimestamp).." sec")
    end
  end
  return '\n'..table.concat(out,'\n')..'\n'
end

local function fakeLastComTimestamp()
  local rootSes = S[0].ROOT_SESSION
  if rootSes then
    S[rootSes].lastComTimestamp = time() - 
    (tonumber(getConfOrDefault{'system', 'sessions', 'idle-time'}) or 5*60)
  end
end

local function closeOtherSessions(sesNum, sesNumClosed)
  if S[sesNum].num ~= sesNum or S[sesNumClosed].num ~= sesNumClosed then
    return "Bad session"
  end
  if sesNum == sesNumClosed then
    return S[sesNum]:closeSession()
  end
  local perm = users[S[sesNum].user].permission
  if perm ~= 'rw' then
    return ("You have no permission to close session %d"):format(sesNumClosed)
  else
    local hisPerm = users[S[sesNumClosed].user].permission
    if hisPerm == 'rw' and S[sesNum].user ~= 'root' then
      return ("Only root can close session %d"):format(sesNumClosed)
    else
      return S[sesNumClosed]:closeSession()
    end
  end
  return'OK'
end

--------------------------------------------------------------------------------
-- Watchdog
local WD
local wdFirstApply
local softWatchdogSock
local function watchdog(enable)
  if enable == 'show' then
    return '\n'..(WD and tostring(WD) or "watchdog: disabled")..'\n'
  end
  if enable and not WD then
    if wdFirstApply == true then
      wdFirstApply = "start"
      return
    end
    log_info("Start warchdog")
    WD = nsg.watchdog()
    WD:settimeout(30)

    softWatchdogSock = socket.udp()
    softWatchdogSock:setpeername('127.0.0.1', 50000)
    os.execute"/usr/sbin/nsgsoftwatchdog"

  elseif WD and not enable then
    log_info("Stop watchdog")
    softWatchdogSock:send'CLOSE'
    softWatchdogSock:close()
    WD:close()
    WD = nil
  end
end


-- handlers
local function findDelim(buf)
  local s
  local e = 0
  repeat
    s, e = buf:find("\\*"..MESSAGE_DELIMITER, e + 1)
    if s and (e-s)%2==0 then
      return e
    end
  until not s
  return nil
end

local function pcallToStringTable(...)
  local t = {...}
  local n = select('#', ...)
  local f = t[1] and 1 or 0
  for i = 1, n - f do
    t[i] = toStr(t[i + f])
  end
  t[n + 1 - f] = nil
  return t
end

local function comHandler(skt)
  skt = myWrap(skt)
  local buf, command = "", ""
  while true do
    local data, err = skt:rcv(2048)
    if not data then return end
    buf = buf..data
    while true do
      local d = findDelim(buf)
      if not d then break end
      command = buf:sub(1, d-1):gsub("\\(.)", "%1")
      log_debug('command: %s', command)
      buf = buf:sub(d + 1)

      local sesNum, com = command:match"^%s*S%[(%d+)%]:(.-)%(.*%)%s*$"
      sesNum = tonumber(sesNum)
      local res
      if sesNum then
        if com ~= 'tty' then
          S[sesNum].lastComTimestamp = time()
        end
        local request, err = loadstring('return '..command)
        if not request then
          res = pcallToStringTable(request, err)
        else
          setfenv(request, {S=S})
          C.session = S[sesNum]
          res = pcallToStringTable(countedPcall(request))
          C.session = nil
        end
      else
        res = pcallToStringTable(false, 'Bad command syntax')
      end
      local mes = ""
      for _, v in ipairs(res) do 
        mes = mes..v:sub(1,20):gsub('\27', '<ESC>')..', ' 
      end
      log_debug('reply: %s', mes)
      skt:snd(concat(res,','))
    end
  end
end


-- initial scripts
do
  local year = tonumber(scheme.readpopen"date +%Y")
  if not year or year < 2010 then
    os.execute"date 010100002010"
  end
end

os.execute"touch /var/log/lastlog;chgrp utmp /var/log/lastlog;chmod 664 /var/log/lastlog"


if scheme.readpopen('/usr/sbin/ip addr show dev eth0 2>&1'):match"%s+inet%s" 
then -- eth0 already has IP address(by BOOTP or other way)
  setConfigValue({'port', 'eth0', 'ifAddress', 'configurable'}, "no")
else
  if not getConfOrDefault{'port', 'eth0', 'ifAddress', 'prefix'} then
    setConfigValue({'port', 'eth0', 'ifAddress', 'prefix'}, '192.168.1.1/24')
    setConfigValue({'port', 'eth0', 'link', 'adm-state'}, 'up')
  end
end


-- populate enviroment for scheme modules
C = {
  getSchemeByPath = getSchemeByPath,
  getConfByPath = getConfByPath,
  getConfOrDefault = getConfOrDefault,
  getConfWithDefault = getConfWithDefault,
  setConfigValue = setConfigValue,
  addUser = addUser,
  updateRootNsgUser = updateRootNsgUser,
  restartDaemon = restartDaemon,
  getDaemonLog = getDaemonLog,
  getDaemonStat = getDaemonStat,
  cleanDaemonLog = cleanDaemonLog,
  threadSleep = threadSleep,
  increasingSleep = increasingSleep,
  showSessions = showSessions,
  closeOtherSessions = closeOtherSessions,
  fakeLastComTimestamp = fakeLastComTimestamp,
  log_debug = log_debug,
  log_info = log_info,
  log_warn = log_warn,
  log_error = log_error,
  log_fatal = log_fatal,
  timer = timer,
  getSessionInfo = getSessionInfo,
  watchdog = watchdog,
  setOption = setOption,
  getOption = getOption,
  applyConf = applyConfAux,
  removeConfigValue = removeConfigValue,
  insertKey = insertKeyCommon,
  confToTable = userConfToTable,
  loadConfig = loadConfig,
  addNeteventCallback = addNeteventCallback,
  deleteNeteventCallback = deleteNeteventCallback,
  updateNeteventCallback = updateNeteventCallback,
  BI = BI,
  setLogLevel = setLogLevel,
  setLogMaxFilesize = setLogMaxFilesize,
  getVersion = getVersion,
}

-- Aplly configuration
--applyConfAux({}, true)

-- netlink listener
copas.addthread(netevent)

wdFirstApply = true
local res, err, warn =  pcall(applyConfAux, {}, true)
if not res then
  log_error("Can't apply config: %s", err)
  initApplyError = (initApplyError or '')..err..'\n'
elseif not err then
  --log_error("Initial config apply: %s", warn)
  initApplyError = (initApplyError or '')..warn..'\n'
end

-- Safe mode
--print(s.ser(BI, 0))
if BI.env and BI.env.safe_mode then
  nsgConfig.configFile = '/etc/etc.sav/nsgconfig'
  loadConfig()
  initMesages = "*** You are in safe mode ***\n"
end

-- Add main server
local server = assert(socket.bind("127.0.0.1", 50007))
copas.addserver(server, comHandler)

-- netlink listener
--copas.addthread(netevent)

if wdFirstApply == "start" then
  wdFirstApply = nil
  watchdog(true)
end
wdFirstApply = nil

while true do
  copas.step(timer:nextTmo(5))
  timer:handler()
  if WD then 
    softWatchdogSock:send'1'
    WD:keepalive() 
  end
  --collectgarbage"collect"  print('#####', collectgarbage"count")
end

